@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2025. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -520,7 +520,7 @@ handle_error(Process* c_p, ErtsCodePtr pc, Eterm* reg,
     c_p->fvalue = NIL;
 
     /* Find a handler or die */
-    if ((c_p->catches > 0 || IS_TRACED_FL(c_p, F_EXCEPTION_TRACE))
+    if ((c_p->catches > 0 || c_p->return_trace_frames > 0)
 	&& !(c_p->freason & EXF_PANIC)) {
 	ErtsCodePtr new_pc;
         /* The Beam handler code (catch_end or try_end) checks reg[0]
@@ -555,10 +555,6 @@ handle_error(Process* c_p, ErtsCodePtr pc, Eterm* reg,
 #else
             /* To avoid keeping stale references. */
             c_p->stop[0] = NIL;
-#endif
-#ifdef ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-	    /* No longer safe to use this position */
-            erts_msgq_recv_marker_clear(c_p, erts_old_recv_marker_id);
 #endif
             c_p->ftrace = NIL;
 	    return new_pc;
@@ -579,21 +575,12 @@ next_catch(Process* c_p, Eterm *reg) {
     int active_catches = c_p->catches > 0;
     ErtsCodePtr return_to_trace_address = NULL;
     int have_return_to_trace = 0;
+    Eterm session_weak_id = NIL;
     Eterm *ptr, *prev;
     ErtsCodePtr handler;
 
     ptr = prev = c_p->stop;
-    ASSERT(ptr <= STACK_START(c_p));
-
-    /* This function is only called if we have active catch tags or have
-     * previously called a function that was exception-traced. As the exception
-     * trace flag isn't cleared after the traced function returns (and the
-     * catch tag inserted by it is gone), it's possible to land here with an
-     * empty stack, and the process should simply die when that happens. */
-    if (ptr == STACK_START(c_p)) {
-        ASSERT(!active_catches && IS_TRACED_FL(c_p, F_EXCEPTION_TRACE));
-        return NULL;
-    }
+    ASSERT(ptr < STACK_START(c_p));
 
     while (ptr < STACK_START(c_p)) {
         Eterm val = ptr[0];
@@ -613,20 +600,22 @@ next_catch(Process* c_p, Eterm *reg) {
 
             if (BeamIsReturnTrace(return_address)) {
                 if (return_address == beam_exception_trace) {
-                    ErtsTracer *tracer;
                     ErtsCodeMFA *mfa;
 
                     mfa = (ErtsCodeMFA*)cp_val(frame[0]);
-                    tracer = ERTS_TRACER_FROM_ETERM(&frame[1]);
 
                     ASSERT_MFA(mfa);
-                    erts_trace_exception(c_p, mfa, reg[3], reg[1], tracer);
+                    erts_trace_exception(c_p, mfa, reg[3], reg[1], frame[1], frame[2]);
                 }
+                ASSERT(c_p->return_trace_frames > 0);
+                c_p->return_trace_frames--;
+
                 ptr += CP_SIZE + BEAM_RETURN_TRACE_FRAME_SZ;
             } else if (BeamIsReturnCallAccTrace(return_address)) {
                 ptr += CP_SIZE + BEAM_RETURN_CALL_ACC_TRACE_FRAME_SZ;
             } else if (BeamIsReturnToTrace(return_address)) {
                 have_return_to_trace = 1; /* Record next cp */
+                session_weak_id = frame[0];
                 return_to_trace_address = NULL;
 
                 ptr += CP_SIZE + BEAM_RETURN_TO_TRACE_FRAME_SZ;
@@ -654,13 +643,17 @@ next_catch(Process* c_p, Eterm *reg) {
     ASSERT(ptr < STACK_START(c_p));
     c_p->stop = prev;
 
-    if (IS_TRACED_FL(c_p, F_TRACE_RETURN_TO) && return_to_trace_address) {
-        /* The stackframe closest to the catch contained an
-         * return_to_trace entry, so since the execution now
-         * continues after the catch, a return_to trace message
-         * would be appropriate.
-         */
-        erts_trace_return_to(c_p, return_to_trace_address);
+    if (return_to_trace_address) {
+        ErtsTracerRef *ref = get_tracer_ref_from_weak_id(&c_p->common,
+                                                         session_weak_id);
+        if (ref && IS_SESSION_TRACED_FL(ref, F_TRACE_RETURN_TO)) {
+            /* The stackframe closest to the catch contained an
+             * return_to_trace entry, so since the execution now
+             * continues after the catch, a return_to trace message
+             * would be appropriate.
+             */
+            erts_trace_return_to(c_p, return_to_trace_address, session_weak_id);
+        }
     }
 
     /* Clear the try_tag or catch_tag in the stack frame so that we
@@ -952,6 +945,9 @@ save_stacktrace(Process* c_p, ErtsCodePtr pc, Eterm* reg,
             break;
 
             /* Kernel */
+        case am_code:
+            format_module = am_erl_kernel_errors;
+            break;
         case am_os:
             format_module = am_erl_kernel_errors;
             break;
@@ -1322,6 +1318,46 @@ call_error_handler(Process* p, const ErtsCodeMFA *mfa, Eterm* reg, Eterm func)
     return ep;
 }
 
+static Export*
+apply_setup_error_handler(Process* p, Eterm module, Eterm function, Uint arity, Eterm* reg)
+{
+    Export* ep;
+
+    /*
+     * Find the export table index for the error handler. Return NULL if
+     * there is no error handler module.
+     */
+
+    if ((ep = erts_active_export_entry(erts_proc_get_error_handler(p),
+				     am_undefined_function, 3)) == NULL) {
+	return NULL;
+    } else {
+	int i;
+	Uint sz = 2*arity;
+	Eterm* hp;
+	Eterm args = NIL;
+
+	/*
+	 * Always copy args from registers to a new list; this ensures
+	 * that we have the same behaviour whether or not this was
+	 * called from apply or fixed_apply (any additional last
+	 * THIS-argument will be included, assuming that arity has been
+	 * properly adjusted).
+	 */
+
+        hp = HAlloc(p, sz);
+	for (i = arity-1; i >= 0; i--) {
+	    args = CONS(hp, reg[i], args);
+	    hp += 2;
+	}
+	reg[0] = module;
+	reg[1] = function;
+	reg[2] = args;
+    }
+
+    return ep;
+}
+
 static ERTS_INLINE void
 apply_bif_error_adjustment(Process *p, Export *ep,
                            Eterm *reg, Uint arity,
@@ -1496,13 +1532,18 @@ apply(Process* p, Eterm* reg, ErtsCodePtr I, Uint stack_offset)
 	goto error;
     }
 
-    /* Call the referenced function, if any: should the function not be found,
-     * create a stub entry which in turn calls the error handler. */
-    ep = erts_export_get_or_make_stub(module, function, arity);
+    /*
+     * Get the index into the export table, or failing that the export
+     * entry for the error handler.
+     *
+     * Note: All BIFs have export entries; thus, no special case is needed.
+     */
 
+    if ((ep = erts_active_export_entry(module, function, arity)) == NULL) {
+	if ((ep = apply_setup_error_handler(p, module, function, arity, reg)) == NULL) goto error;
+    }
     apply_bif_error_adjustment(p, ep, reg, arity, I, stack_offset);
     DTRACE_GLOBAL_CALL_FROM_EXPORT(p, ep);
-
     return ep;
 }
 
@@ -1537,9 +1578,17 @@ fixed_apply(Process* p, Eterm* reg, Uint arity,
 	return apply(p, reg, I, stack_offset);
     }
 
-    /* Call the referenced function, if any: should the function not be found,
-     * create a stub entry which in turn calls the error handler. */
-    ep = erts_export_get_or_make_stub(module, function, arity);
+    /*
+     * Get the index into the export table, or failing that the export
+     * entry for the error handler module.
+     *
+     * Note: All BIFs have export entries; thus, no special case is needed.
+     */
+
+    if ((ep = erts_active_export_entry(module, function, arity)) == NULL) {
+	if ((ep = apply_setup_error_handler(p, module, function, arity, reg)) == NULL)
+	    goto error;
+    }
 
     apply_bif_error_adjustment(p, ep, reg, arity, I, stack_offset);
     DTRACE_GLOBAL_CALL_FROM_EXPORT(p, ep);
@@ -1626,6 +1675,7 @@ erts_hibernate(Process* c_p, Eterm* reg)
     }
 
     c_p->catches = 0;
+    c_p->return_trace_frames = 0;
     c_p->i = beam_run_process;
 
     /*
@@ -1676,8 +1726,14 @@ call_fun(Process* p,    /* Current process. */
     code_ix = erts_active_code_ix();
     code_ptr = (funp->entry.disp)->addresses[code_ix];
 
-    if (ERTS_LIKELY(code_ptr != beam_unloaded_fun && funp->arity == arity)) {
-        for (int i = 0, num_free = funp->num_free; i < num_free; i++) {
+    if (ERTS_LIKELY(code_ptr != beam_unloaded_fun &&
+                    fun_arity(funp) == arity)) {
+        /* Copy the free variables, skipping the FunRef in the environment.
+         *
+         * Note that we avoid using fun_num_free as it asserts that the
+         * argument is a local function, which we don't need to care about
+         * here. */
+        for (int i = 0, num_free = fun_env_size(funp) - 1; i < num_free; i++) {
             reg[i + arity] = funp->env[i];
         }
 
@@ -1686,7 +1742,6 @@ call_fun(Process* p,    /* Current process. */
             DTRACE_LOCAL_CALL(p, erts_code_to_codemfa(code_ptr));
         } else {
             Export *ep = funp->entry.exp;
-            ASSERT(is_external_fun(funp) && funp->next == NULL);
             DTRACE_GLOBAL_CALL(p, &ep->info.mfa);
         }
 #endif
@@ -1716,7 +1771,7 @@ call_fun(Process* p,    /* Current process. */
             }
         }
 
-        if (funp->arity != arity) {
+        if (fun_arity(funp) != arity) {
             /* There is a fun defined, but the call has the wrong arity. */
             Eterm *hp = HAlloc(p, 3);
             p->freason = EXC_BADARITY;
@@ -1816,7 +1871,7 @@ is_function2(Eterm Term, Uint arity)
 {
     if (is_any_fun(Term)) {
         ErlFunThing *funp = (ErlFunThing*)fun_val(Term);
-        return funp->arity == arity;
+        return fun_arity(funp) == arity;
     }
 
     return 0;
@@ -1824,7 +1879,7 @@ is_function2(Eterm Term, Uint arity)
 
 Eterm get_map_element(Eterm map, Eterm key)
 {
-    Uint32 hx;
+    erts_ihash_t hx;
     const Eterm *vs;
     if (is_flatmap(map)) {
 	flatmap_t *mp;
@@ -1857,7 +1912,7 @@ Eterm get_map_element(Eterm map, Eterm key)
     return vs ? *vs : THE_NON_VALUE;
 }
 
-Eterm get_map_element_hash(Eterm map, Eterm key, Uint32 hx)
+Eterm get_map_element_hash(Eterm map, Eterm key, erts_ihash_t hx)
 {
     const Eterm *vs;
 
@@ -2029,7 +2084,7 @@ erts_gc_update_map_assoc(Process* p, Eterm* reg, Uint live,
     map = reg[live];
 
     if (is_not_flatmap(map)) {
-	Uint32 hx;
+	erts_ihash_t hx;
 	Eterm val;
 
 	ASSERT(is_hashmap(map));
@@ -2272,7 +2327,7 @@ erts_gc_update_map_exact(Process* p, Eterm* reg, Uint live,
     map = reg[live];
 
     if (is_not_flatmap(map)) {
-	Uint32 hx;
+	erts_ihash_t hx;
 	Eterm val;
 
 	/* apparently the compiler does not emit is_map instructions,
