@@ -25,8 +25,6 @@
 
 void erts_term_init(void);
 
-typedef UWord Wterm;  /* Full word terms */
-
 struct erl_node_; /* Declared in erl_node_tables.h */
 
 /*
@@ -92,25 +90,27 @@ struct erl_node_; /* Declared in erl_node_tables.h */
 /*
  * HEADER representation:
  *
- *	aaaaaaaaaaaaaaaaaaaaaaaaaatttt00	arity:26, tag:4
+ *      aaaaaaaaaaaaaaaaaaaaaaaaaattttpp          arity:26, tag:4, ptag:2
+ *
+ * Where ptag is always TAG_PRIMARY_HEADER.
  *
  * HEADER tags:
  *
- *	0000	ARITYVAL
- *      0001    BINARY_AGGREGATE                |
- *	001x	BIGNUM with sign bit		|
- *	0100	REF				|
- *	0101	FUN				| THINGS
- *	0110	FLONUM				|
- *      0111    EXPORT                          |
- *	1000	REFC_BINARY	|		|
- *	1001	HEAP_BINARY	| BINARIES	|
- *	1010	SUB_BINARY	|		|
- *      1011    Not used; see comment below
- *      1100    EXTERNAL_PID  |                 |
- *      1101    EXTERNAL_PORT | EXTERNAL THINGS |
- *      1110    EXTERNAL_REF  |                 |
- *      1111    MAP
+ *      0000    ARITYVAL
+ *      0001    FUN_REF                           |
+ *      001x    BIGNUM with sign bit              |
+ *      0100    REF                               |
+ *      0101    FUN                               | THINGS
+ *      0110    FLONUM                            |
+ *      0111    -- FREE --                        |
+ *      1000    HEAP_BITS     | BITSTRINGS        |
+ *      1001    SUB_BITS      |                   |
+ *      1010    BIN_REF	                          |
+ *      1011    MAP                               |
+ *      1100    EXTERNAL_PID  | EXTERNAL THINGS   |
+ *      1101    EXTERNAL_PORT |                   |
+ *      1110    EXTERNAL_REF  |                   |
+ *      1111    -- FREE --    | Reserved external |
  *
  * COMMENTS:
  *
@@ -121,38 +121,39 @@ struct erl_node_; /* Declared in erl_node_tables.h */
  *
  * XXX: globally replace XXX_SUBTAG with TAG_HEADER_XXX
  */
-#define ARITYVAL_SUBTAG		(0x0 << _TAG_PRIMARY_SIZE) /* TUPLE */
-#define BIN_MATCHSTATE_SUBTAG	(0x1 << _TAG_PRIMARY_SIZE) 
-#define POS_BIG_SUBTAG		(0x2 << _TAG_PRIMARY_SIZE) /* BIG: tags 2&3 */
-#define NEG_BIG_SUBTAG		(0x3 << _TAG_PRIMARY_SIZE) /* BIG: tags 2&3 */
-#define _BIG_SIGN_BIT		(0x1 << _TAG_PRIMARY_SIZE)
-#define REF_SUBTAG		(0x4 << _TAG_PRIMARY_SIZE) /* REF */
-#define FUN_SUBTAG		(0x5 << _TAG_PRIMARY_SIZE) /* FUN */
-#define FLOAT_SUBTAG		(0x6 << _TAG_PRIMARY_SIZE) /* FLOAT */
-#define _BINARY_XXX_MASK	(0x3 << _TAG_PRIMARY_SIZE)
-#define REFC_BINARY_SUBTAG	(0x8 << _TAG_PRIMARY_SIZE) /* BINARY */
-#define HEAP_BINARY_SUBTAG	(0x9 << _TAG_PRIMARY_SIZE) /* BINARY */
-#define SUB_BINARY_SUBTAG	(0xA << _TAG_PRIMARY_SIZE) /* BINARY */
-/*   _BINARY_XXX_MASK depends on 0xB being unused */
-#define EXTERNAL_PID_SUBTAG	(0xC << _TAG_PRIMARY_SIZE) /* EXTERNAL_PID */
-#define EXTERNAL_PORT_SUBTAG	(0xD << _TAG_PRIMARY_SIZE) /* EXTERNAL_PORT */
-#define EXTERNAL_REF_SUBTAG	(0xE << _TAG_PRIMARY_SIZE) /* EXTERNAL_REF */
-#define MAP_SUBTAG		(0xF << _TAG_PRIMARY_SIZE) /* MAP */
-
+#define ARITYVAL_SUBTAG         (0x0 << _TAG_PRIMARY_SIZE) /* TUPLE */
+#define FUN_REF_SUBTAG          (0x1 << _TAG_PRIMARY_SIZE) /* FUN_REF */
+#define _BIG_TAG_MASK           (~(0x1 << _TAG_PRIMARY_SIZE) & _TAG_HEADER_MASK)
+#define _BIG_SIGN_BIT           (0x1 << _TAG_PRIMARY_SIZE)
+#define POS_BIG_SUBTAG          (0x2 << _TAG_PRIMARY_SIZE) /* BIGNUM */
+#define NEG_BIG_SUBTAG          (0x3 << _TAG_PRIMARY_SIZE) /* BIGNUM */
+#define REF_SUBTAG              (0x4 << _TAG_PRIMARY_SIZE) /* REF */
+#define FUN_SUBTAG              (0x5 << _TAG_PRIMARY_SIZE) /* FUN */
+#define FLOAT_SUBTAG            (0x6 << _TAG_PRIMARY_SIZE) /* FLOAT */
+#define _BITSTRING_TAG_MASK     (~(0x1 << _TAG_PRIMARY_SIZE) & _TAG_HEADER_MASK)
+#define HEAP_BITS_SUBTAG        (0x8 << _TAG_PRIMARY_SIZE) /* BITSTRING */
+#define SUB_BITS_SUBTAG         (0x9 << _TAG_PRIMARY_SIZE) /* BITSTRING */
+#define BIN_REF_SUBTAG          (0xA << _TAG_PRIMARY_SIZE)
+#define MAP_SUBTAG              (0xB << _TAG_PRIMARY_SIZE) /* MAP */
+#define _EXTERNAL_TAG_MASK      (~(0x3 << _TAG_PRIMARY_SIZE) & _TAG_HEADER_MASK)
+#define EXTERNAL_PID_SUBTAG     (0xC << _TAG_PRIMARY_SIZE) /* EXTERNAL_PID */
+#define EXTERNAL_PORT_SUBTAG    (0xD << _TAG_PRIMARY_SIZE) /* EXTERNAL_PORT */
+#define EXTERNAL_REF_SUBTAG     (0xE << _TAG_PRIMARY_SIZE) /* EXTERNAL_REF */
+/* _EXTERNAL_TAG_MASK requires that 0xF is reserved for external terms. */
 
 #define _TAG_HEADER_ARITYVAL       (TAG_PRIMARY_HEADER|ARITYVAL_SUBTAG)
-#define _TAG_HEADER_FUN	           (TAG_PRIMARY_HEADER|FUN_SUBTAG)
+#define _TAG_HEADER_FUN_REF        (TAG_PRIMARY_HEADER|FUN_REF_SUBTAG)
+#define _TAG_HEADER_FUN            (TAG_PRIMARY_HEADER|FUN_SUBTAG)
 #define _TAG_HEADER_POS_BIG        (TAG_PRIMARY_HEADER|POS_BIG_SUBTAG)
 #define _TAG_HEADER_NEG_BIG        (TAG_PRIMARY_HEADER|NEG_BIG_SUBTAG)
 #define _TAG_HEADER_FLOAT          (TAG_PRIMARY_HEADER|FLOAT_SUBTAG)
 #define _TAG_HEADER_REF            (TAG_PRIMARY_HEADER|REF_SUBTAG)
-#define _TAG_HEADER_REFC_BIN       (TAG_PRIMARY_HEADER|REFC_BINARY_SUBTAG)
-#define _TAG_HEADER_HEAP_BIN       (TAG_PRIMARY_HEADER|HEAP_BINARY_SUBTAG)
-#define _TAG_HEADER_SUB_BIN        (TAG_PRIMARY_HEADER|SUB_BINARY_SUBTAG)
+#define _TAG_HEADER_BIN_REF        (TAG_PRIMARY_HEADER|BIN_REF_SUBTAG)
+#define _TAG_HEADER_HEAP_BITS      (TAG_PRIMARY_HEADER|HEAP_BITS_SUBTAG)
+#define _TAG_HEADER_SUB_BITS       (TAG_PRIMARY_HEADER|SUB_BITS_SUBTAG)
 #define _TAG_HEADER_EXTERNAL_PID   (TAG_PRIMARY_HEADER|EXTERNAL_PID_SUBTAG)
 #define _TAG_HEADER_EXTERNAL_PORT  (TAG_PRIMARY_HEADER|EXTERNAL_PORT_SUBTAG)
 #define _TAG_HEADER_EXTERNAL_REF   (TAG_PRIMARY_HEADER|EXTERNAL_REF_SUBTAG)
-#define _TAG_HEADER_BIN_MATCHSTATE (TAG_PRIMARY_HEADER|BIN_MATCHSTATE_SUBTAG)
 #define _TAG_HEADER_MAP	           (TAG_PRIMARY_HEADER|MAP_SUBTAG)
 
 
@@ -164,7 +165,6 @@ struct erl_node_; /* Declared in erl_node_tables.h */
  (((x) & (_HEADER_SUBTAG_MASK)) == ARITYVAL_SUBTAG)
 #define header_is_arityval(x)	(((x) & _HEADER_SUBTAG_MASK) == ARITYVAL_SUBTAG)
 #define header_is_thing(x)	(!header_is_transparent((x)))
-#define header_is_bin_matchstate(x)	((((x) & (_HEADER_SUBTAG_MASK)) == BIN_MATCHSTATE_SUBTAG))
 
 #define _CPMASK		0x3
 
@@ -204,7 +204,7 @@ _ET_DECLARE_CHECKED(int,is_boxed,Eterm)
 #else
 #define _unchecked_boxed_val(x) ((Eterm*) ((x) - TAG_PRIMARY_BOXED))
 #endif
-_ET_DECLARE_CHECKED(Eterm*,boxed_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,boxed_val,Eterm)
 #define boxed_val(x)		_ET_APPLY(boxed_val,(x))
 
 /* cons cell ("list") access methods */
@@ -226,7 +226,7 @@ _ET_DECLARE_CHECKED(int,is_not_list,Eterm)
 #else
 #define _unchecked_list_val(x) ((Eterm*) ((x) - TAG_PRIMARY_LIST))
 #endif
-_ET_DECLARE_CHECKED(Eterm*,list_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,list_val,Eterm)
 #define list_val(x)		_ET_APPLY(list_val,(x))
 
 #define CONS(hp, car, cdr) \
@@ -299,8 +299,10 @@ _ET_DECLARE_CHECKED(Uint,atom_val,Eterm)
 /* header (arityval or thing) access methods */
 #define _make_header(sz,tag) ((Uint)(((Uint)(sz) << _HEADER_ARITY_OFFS) + (tag)))
 #define is_header(x)	(((x) & _TAG_PRIMARY_MASK) == TAG_PRIMARY_HEADER)
-#define _unchecked_header_arity(x) \
-    (is_map_header(x) ? MAP_HEADER_ARITY(x) : ((x) >> _HEADER_ARITY_OFFS))
+#define _unchecked_header_arity(x)                                            \
+    (is_map_header(x) ? MAP_HEADER_ARITY(x) :                                 \
+     (is_fun_header(x) ? (ERL_FUN_SIZE - 1) :                                 \
+      ((x) >> _HEADER_ARITY_OFFS)))
 _ET_DECLARE_CHECKED(Uint,header_arity,Eterm)
 #define header_arity(x)	_ET_APPLY(header_arity,(x))
 
@@ -371,35 +373,62 @@ _ET_DECLARE_CHECKED(Uint,thing_subtag,Eterm)
 #define is_value(x)	((x) != THE_NON_VALUE)
 
 /* binary object access methods */
-#define is_binary_header(x) \
-	((((x) & (_TAG_HEADER_MASK)) == _TAG_HEADER_REFC_BIN) || \
-	 (((x) & (_TAG_HEADER_MASK)) == _TAG_HEADER_HEAP_BIN) || \
-	 (((x) & (_TAG_HEADER_MASK)) == _TAG_HEADER_SUB_BIN))
+#define is_bitstring_header(x)                                                \
+    (((x) & (_BITSTRING_TAG_MASK)) == _TAG_HEADER_HEAP_BITS)
 
-#define make_binary(x)	make_boxed((Eterm*)(x))
-#define is_binary(x)	(is_boxed((x)) && is_binary_header(*boxed_val((x))))
-#define is_not_binary(x) (!is_binary((x)))
-#define _unchecked_binary_val(x) _unchecked_boxed_val((x))
-_ET_DECLARE_CHECKED(Eterm*,binary_val,Wterm)
-#define binary_val(x)	_ET_APPLY(binary_val,(x))
+#define is_bin_ref(x)                                                         \
+    (is_boxed((x)) && *boxed_val(x) == HEADER_BIN_REF)
+#define make_bitstring(x)	make_boxed((Eterm*)(x))
+#define is_bitstring(x)	(is_boxed((x)) && is_bitstring_header(*boxed_val((x))))
+#define is_not_bitstring(x) (!is_bitstring((x)))
+#define _unchecked_bitstring_val(x) _unchecked_boxed_val((x))
+_ET_DECLARE_CHECKED(Eterm*,bitstring_val,Eterm)
+#define bitstring_val(x)	_ET_APPLY(bitstring_val,(x))
 
-/* process binaries stuff (special case of binaries) */
-#define HEADER_PROC_BIN	_make_header(PROC_BIN_SIZE-1,_TAG_HEADER_REFC_BIN)
+/* Fun objects.
+ *
+ * These have a special tag scheme to make the representation as compact as
+ * possible. For normal headers, we have:
+ *
+ *     aaaaaaaaaaaaaaaa aaaaaaaaaatttt00       arity:26, tag:4
+ *
+ * Since the arity and number of free variables are both limited to 255, we can
+ * fit them both into the header word.
+ *
+ *     0000000eeeeeeeee aaaaaaaa00010100       environment_size:9, arity:8
+ *
+ * Note that the lowest byte contains only the function subtag, and the next
+ * byte after that contains only the arity. This lets us combine the type
+ * and/or arity check into a single comparison without masking, by using 8- or
+ * 16-bit operations on the header word.
+ *
+ * Furthermore, as local funs always have at least one environment variable as
+ * the `FunRef` is treated as part of the environment, we can distinguish
+ * them by checking whether the environment size is zero. */
 
-/* fun objects */
-#define HEADER_FUN              _make_header(ERL_FUN_SIZE-2,_TAG_HEADER_FUN)
-#define is_fun_header(x)        ((x) == HEADER_FUN)
+#define FUN_HEADER_ARITY_OFFS (_HEADER_ARITY_OFFS + 2)
+#define FUN_HEADER_ENV_SIZE_OFFS (FUN_HEADER_ARITY_OFFS + 8)
+
+#define MAKE_FUN_HEADER(Arity, NumFree, External)                             \
+    (ASSERT((!(External)) || ((NumFree) == 0)),                               \
+     (_TAG_HEADER_FUN |                                                       \
+     (((Arity)) << FUN_HEADER_ARITY_OFFS) |                                   \
+     (((NumFree + !External)) << FUN_HEADER_ENV_SIZE_OFFS)))
+
+#define is_fun_ref(x)                                                         \
+    (is_boxed((x)) && *boxed_val(x) == HEADER_FUN_REF)
+#define is_fun_header(x)        (((x) & _HEADER_SUBTAG_MASK) == FUN_SUBTAG)
 #define make_fun(x)             make_boxed((Eterm*)(x))
 #define is_any_fun(x)           (is_boxed((x)) && is_fun_header(*boxed_val((x))))
 #define is_not_any_fun(x)       (!is_any_fun((x)))
 #define _unchecked_fun_val(x)   _unchecked_boxed_val((x))
-_ET_DECLARE_CHECKED(Eterm*,fun_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,fun_val,Eterm)
 #define fun_val(x)		_ET_APPLY(fun_val,(x))
 
 /* bignum access methods */
 #define make_pos_bignum_header(sz)	_make_header((sz),_TAG_HEADER_POS_BIG)
 #define make_neg_bignum_header(sz)	_make_header((sz),_TAG_HEADER_NEG_BIG)
-#define _is_bignum_header(x)	(((x) & (_TAG_HEADER_MASK-_BIG_SIGN_BIT)) == _TAG_HEADER_POS_BIG)
+#define _is_bignum_header(x)	(((x) & _BIG_TAG_MASK) == _TAG_HEADER_POS_BIG)
 #define _unchecked_bignum_header_is_neg(x)	((x) & _BIG_SIGN_BIT)
 _ET_DECLARE_CHECKED(int,bignum_header_is_neg,Eterm)
 #define bignum_header_is_neg(x)	_ET_APPLY(bignum_header_is_neg,(x))
@@ -419,7 +448,7 @@ _ET_DECLARE_CHECKED(Uint,bignum_header_arity,Eterm)
 #define is_big(x)	(is_boxed((x)) && _is_bignum_header(*boxed_val((x))))
 #define is_not_big(x)	(!is_big((x)))
 #define _unchecked_big_val(x)	_unchecked_boxed_val((x))
-_ET_DECLARE_CHECKED(Eterm*,big_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,big_val,Eterm)
 #define big_val(x)		_ET_APPLY(big_val,(x))
 
 /* flonum ("float") access methods */
@@ -432,7 +461,7 @@ _ET_DECLARE_CHECKED(Eterm*,big_val,Wterm)
 #define is_float(x)	(is_boxed((x)) && *boxed_val((x)) == HEADER_FLONUM)
 #define is_not_float(x)	(!is_float(x))
 #define _unchecked_float_val(x)	_unchecked_boxed_val((x))
-_ET_DECLARE_CHECKED(Eterm*,float_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,float_val,Eterm)
 #define float_val(x)	_ET_APPLY(float_val,(x))
 
 /* Float definition for byte and word access */
@@ -444,9 +473,7 @@ typedef union float_def
     byte   fb[sizeof(ieee754_8)];
     Uint16 fs[sizeof(ieee754_8) / sizeof(Uint16)];
     Uint32 fw[sizeof(ieee754_8) / sizeof(Uint32)];
-#if defined(ARCH_64)
-    Uint   fdw;
-#endif
+    Uint64 fdw;
 } FloatDef;
 
 #if defined(ARCH_64)
@@ -483,7 +510,7 @@ typedef union float_def
    (is_boxed((x)) && *boxed_val((x)) == make_arityval_unchecked((a)))
 #define is_not_tuple_arity(x, a) (!is_tuple_arity((x),(a)))
 #define _unchecked_tuple_val(x)	_unchecked_boxed_val(x)
-_ET_DECLARE_CHECKED(Eterm*,tuple_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,tuple_val,Eterm)
 #define tuple_val(x)	_ET_APPLY(tuple_val,(x))
 
 /*
@@ -1016,12 +1043,12 @@ typedef union {
   (!is_internal_ref((x)))
 
 #define _unchecked_internal_ref_val(x) _unchecked_boxed_val((x))
-_ET_DECLARE_CHECKED(Eterm*,internal_ref_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,internal_ref_val,Eterm)
 #define internal_ref_val(x) _ET_APPLY(internal_ref_val,(x))
 
 #define internal_non_magic_ref_thing_numbers(rt) (((ErtsORefThing *)(rt))->num)
 #define _unchecked_internal_non_magic_ref_numbers(x) (internal_non_magic_ref_thing_numbers(_unchecked_ordinary_ref_thing_ptr(x)))
-_ET_DECLARE_CHECKED(Uint32*,internal_non_magic_ref_numbers,Wterm)
+_ET_DECLARE_CHECKED(Uint32*,internal_non_magic_ref_numbers,Eterm)
 #define internal_non_magic_ref_numbers(x) _ET_APPLY(internal_non_magic_ref_numbers,(x))
 
 #define internal_ordinary_ref_numbers(x) internal_non_magic_ref_numbers((x))
@@ -1034,7 +1061,7 @@ _ET_DECLARE_CHECKED(Uint32*,internal_non_magic_ref_numbers,Wterm)
 #endif
 
 #define _unchecked_internal_magic_ref_numbers(x) (internal_magic_thing_ref_numbers(_unchecked_magic_ref_thing_ptr(x)))
-_ET_DECLARE_CHECKED(Uint32*,internal_magic_ref_numbers,Wterm)
+_ET_DECLARE_CHECKED(Uint32*,internal_magic_ref_numbers,Eterm)
 #define internal_magic_ref_numbers(x) _ET_APPLY(internal_magic_ref_numbers,(x))
 
 
@@ -1136,8 +1163,7 @@ typedef struct external_thing_ {
   (((x) & _TAG_HEADER_MASK) == _TAG_HEADER_EXTERNAL_REF)
 
 #define is_external_header(x) \
-  (((x) & (_TAG_HEADER_MASK-_BINARY_XXX_MASK)) == _TAG_HEADER_EXTERNAL_PID \
-   && ((x) & _TAG_HEADER_MASK) != _TAG_HEADER_MAP)
+  (((x) & (_EXTERNAL_TAG_MASK)) == _TAG_HEADER_EXTERNAL_PID)
 
 #define is_external(x) (is_boxed((x)) && is_external_header(*boxed_val((x))))
 
@@ -1164,7 +1190,7 @@ typedef struct external_thing_ {
 #define make_external_ref		make_external
 
 #define _unchecked_external_val(x) _unchecked_boxed_val((x))
-_ET_DECLARE_CHECKED(Eterm*,external_val,Wterm)
+_ET_DECLARE_CHECKED(Eterm*,external_val,Eterm)
 #define external_val(x) _ET_APPLY(external_val,(x))
 
 #define external_thing_ptr(x) ((ExternalThing *) external_val((x)))
@@ -1173,12 +1199,12 @@ _ET_DECLARE_CHECKED(Eterm*,external_val,Wterm)
 
 #define _unchecked_external_thing_data_words(thing) \
     (_unchecked_thing_arityval((thing)->header) + (1 - EXTERNAL_THING_HEAD_SIZE))
-_ET_DECLARE_CHECKED(Uint,external_thing_data_words,ExternalThing*)
+_ET_DECLARE_CHECKED(Uint,external_thing_data_words,const ExternalThing*)
 #define external_thing_data_words(thing) _ET_APPLY(external_thing_data_words,(thing))
 
 #define _unchecked_external_data_words(x) \
     _unchecked_external_thing_data_words(_unchecked_external_thing_ptr((x)))
-_ET_DECLARE_CHECKED(Uint,external_data_words,Wterm)
+_ET_DECLARE_CHECKED(Uint,external_data_words,Eterm)
 #define external_data_words(x) _ET_APPLY(external_data_words,(x))
 
 #define _unchecked_external_data(x) (_unchecked_external_thing_ptr((x))->data.ui)
@@ -1189,11 +1215,11 @@ _ET_DECLARE_CHECKED(Uint,external_data_words,Wterm)
 
 #define _unchecked_external_pid_data_words(x) \
   _unchecked_external_data_words((x))
-_ET_DECLARE_CHECKED(Uint,external_pid_data_words,Wterm)
+_ET_DECLARE_CHECKED(Uint,external_pid_data_words,Eterm)
 #define external_pid_data_words(x) _ET_APPLY(external_pid_data_words,(x))
 
 #define _unchecked_external_pid_node(x) _unchecked_external_node((x))
-_ET_DECLARE_CHECKED(struct erl_node_*,external_pid_node,Wterm)
+_ET_DECLARE_CHECKED(struct erl_node_*,external_pid_node,Eterm)
 #define external_pid_node(x) _ET_APPLY(external_pid_node,(x))
 
 #define external_pid_number(x) (external_thing_ptr(x)->data.pid.num)
@@ -1201,15 +1227,15 @@ _ET_DECLARE_CHECKED(struct erl_node_*,external_pid_node,Wterm)
 
 #define _unchecked_external_port_data_words(x) \
   _unchecked_external_data_words((x))
-_ET_DECLARE_CHECKED(Uint,external_port_data_words,Wterm)
+_ET_DECLARE_CHECKED(Uint,external_port_data_words,Eterm)
 #define external_port_data_words(x) _ET_APPLY(external_port_data_words,(x))
 
 #define _unchecked_external_port_data(x) _unchecked_external_data((x))
-_ET_DECLARE_CHECKED(Uint*,external_port_data,Wterm)
+_ET_DECLARE_CHECKED(Uint*,external_port_data,Eterm)
 #define external_port_data(x) _ET_APPLY(external_port_data,(x))
 
 #define _unchecked_external_port_node(x) _unchecked_external_node((x))
-_ET_DECLARE_CHECKED(struct erl_node_*,external_port_node,Wterm)
+_ET_DECLARE_CHECKED(struct erl_node_*,external_port_node,Eterm)
 #define external_port_node(x) _ET_APPLY(external_port_node,(x))
 
 #ifdef ARCH_64
@@ -1222,12 +1248,12 @@ _ET_DECLARE_CHECKED(struct erl_node_*,external_port_node,Wterm)
 
 #define _unchecked_external_ref_data_words(x) \
   _unchecked_external_data_words((x))
-_ET_DECLARE_CHECKED(Uint,external_ref_data_words,Wterm)
+_ET_DECLARE_CHECKED(Uint,external_ref_data_words,Eterm)
 #define external_ref_data_words(x) _ET_APPLY(external_ref_data_words,(x))
 #define external_thing_ref_data_words(thing) external_thing_data_words(thing)
 
 #define _unchecked_external_ref_data(x) (_unchecked_external_thing_ptr((x))->data.ui32)
-_ET_DECLARE_CHECKED(Uint32*,external_ref_data,Wterm)
+_ET_DECLARE_CHECKED(Uint32*,external_ref_data,Eterm)
 #define external_ref_data(x) _ET_APPLY(external_ref_data,(x))
 #define external_thing_ref_data(thing) ((thing)->data.ui32)
 
@@ -1382,7 +1408,7 @@ _ET_DECLARE_CHECKED(Uint,loader_y_reg_index,Uint)
  *   of the tag_val_def() function
  */
 
-#define BINARY_DEF		0x0
+#define BITSTRING_DEF		0x0
 #define LIST_DEF		0x1
 #define NIL_DEF			0x2
 #define MAP_DEF			0x3
@@ -1398,14 +1424,15 @@ _ET_DECLARE_CHECKED(Uint,loader_y_reg_index,Uint)
 #define FLOAT_DEF		0xe
 #define BIG_DEF			0xf
 #define SMALL_DEF		0x10
-#define MATCHSTATE_DEF          0x11   /* not a "real" term */
+#define BIN_REF_DEF             0x11   /* not a "real" term */
+#define FUN_REF_DEF             0x12   /* not a "real" term */
 
-#define FIRST_VACANT_TAG_DEF    0x12
+#define FIRST_VACANT_TAG_DEF    0x13
 
 #if ET_DEBUG
-ERTS_GLB_INLINE unsigned tag_val_def(Wterm, const char*, unsigned);
+ERTS_GLB_INLINE unsigned tag_val_def(Eterm, const char*, unsigned);
 #else
-ERTS_GLB_INLINE unsigned tag_val_def(Wterm);
+ERTS_GLB_INLINE unsigned tag_val_def(Eterm);
 #endif
 
 #define not_eq_tags(X,Y)	(tag_val_def((X)) ^ tag_val_def((Y)))
@@ -1439,9 +1466,9 @@ do { \
 #if ERTS_GLB_INLINE_INCL_FUNC_DEF
 
 #if ET_DEBUG
-ERTS_GLB_INLINE unsigned tag_val_def(Wterm x, const char *file, unsigned line)
+ERTS_GLB_INLINE unsigned tag_val_def(Eterm x, const char *file, unsigned line)
 #else
-ERTS_GLB_INLINE unsigned tag_val_def(Wterm x)
+ERTS_GLB_INLINE unsigned tag_val_def(Eterm x)
 #define file __FILE__
 #define line __LINE__
 #endif
@@ -1465,10 +1492,10 @@ ERTS_GLB_INLINE unsigned tag_val_def(Wterm x)
 	    case (_TAG_HEADER_EXTERNAL_PORT >> _TAG_PRIMARY_SIZE):	return EXTERNAL_PORT_DEF;
 	    case (_TAG_HEADER_EXTERNAL_REF >> _TAG_PRIMARY_SIZE):	return EXTERNAL_REF_DEF;
 	    case (_TAG_HEADER_MAP >> _TAG_PRIMARY_SIZE):	return MAP_DEF;
-	    case (_TAG_HEADER_REFC_BIN >> _TAG_PRIMARY_SIZE):	return BINARY_DEF;
-	    case (_TAG_HEADER_HEAP_BIN >> _TAG_PRIMARY_SIZE):	return BINARY_DEF;
-	    case (_TAG_HEADER_SUB_BIN >> _TAG_PRIMARY_SIZE):	return BINARY_DEF;
-	    case (_TAG_HEADER_BIN_MATCHSTATE >> _TAG_PRIMARY_SIZE): return MATCHSTATE_DEF;
+	    case (_TAG_HEADER_BIN_REF >> _TAG_PRIMARY_SIZE):	return BIN_REF_DEF;
+	    case (_TAG_HEADER_FUN_REF >> _TAG_PRIMARY_SIZE):	return FUN_REF_DEF;
+	    case (_TAG_HEADER_HEAP_BITS >> _TAG_PRIMARY_SIZE):	return BITSTRING_DEF;
+	    case (_TAG_HEADER_SUB_BITS >> _TAG_PRIMARY_SIZE):	return BITSTRING_DEF;
 	  }
 
 	  break;
