@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2021-2025. All Rights Reserved.
+ * Copyright Ericsson AB 2021-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -104,9 +104,9 @@ void BeamAssemblerCommon::codegen(JitAllocator *allocator,
     }
 #endif
 
-    err = allocator->alloc(const_cast<void **>(executable_ptr),
-                           writable_ptr,
-                           code.codeSize() + 16);
+    JitAllocator::Span span;
+
+    err = allocator->alloc(span, code.codeSize() + 16);
 
     if (err == ErrorCode::kErrorTooManyHandles) {
         ERTS_ASSERT(!"Failed to allocate module code: "
@@ -115,6 +115,9 @@ void BeamAssemblerCommon::codegen(JitAllocator *allocator,
         ERTS_ASSERT("Failed to allocate module code");
     }
 
+    *executable_ptr = span.rx();
+    *writable_ptr = span.rw();
+
     VirtMem::protectJitMemory(VirtMem::ProtectJitAccess::kReadWrite);
 
     code.relocateToBase((uint64_t)*executable_ptr);
@@ -357,7 +360,7 @@ BeamModuleAssembler::BeamModuleAssembler(BeamGlobalAssembler *_ga,
     }
 }
 
-void BeamModuleAssembler::register_metadata(const BeamCodeHeader *header) {
+void *BeamModuleAssembler::register_metadata(const BeamCodeHeader *header) {
 #ifndef WIN32
     const BeamCodeLineTab *line_table = header->line_table;
 
@@ -472,10 +475,12 @@ void BeamModuleAssembler::register_metadata(const BeamCodeHeader *header) {
              .stop = (ErtsCodePtr)(code.baseAddress() + code.codeSize()),
              .name = module_name + "::codeFooter"});
 
-    beamasm_metadata_update(module_name,
-                            (ErtsCodePtr)code.baseAddress(),
-                            code.codeSize(),
-                            ranges);
+    return beamasm_metadata_insert(module_name,
+                                   (ErtsCodePtr)code.baseAddress(),
+                                   code.codeSize(),
+                                   ranges);
+#else
+    return NULL;
 #endif
 }
 
@@ -569,14 +574,6 @@ void BeamModuleAssembler::patchStrings(char *rw_base,
 
 #if defined(DEBUG) && defined(JIT_HARD_DEBUG)
 void beam_jit_validate_term(Eterm term) {
-    if (is_boxed(term)) {
-        Eterm header = *boxed_val(term);
-
-        if (header_is_bin_matchstate(header)) {
-            return;
-        }
-    }
-
     size_object_x(term, NULL);
 }
 #endif
@@ -744,8 +741,8 @@ Uint beam_jit_get_map_elements(Eterm map,
         ASSERT(is_hashmap(map));
 
         while (n--) {
+            erts_ihash_t hx;
             const Eterm *v;
-            Uint32 hx;
 
             hx = fs[2];
             ASSERT(hx == hashmap_make_hash(fs[0]));
@@ -815,79 +812,43 @@ Eterm beam_jit_bs_init_bits(Process *c_p,
                             Uint num_bits,
                             Uint alloc,
                             unsigned Live) {
-    Eterm new_binary;
-    Uint num_bytes = ((Uint64)num_bits + (Uint64)7) >> 3;
-
-    if (num_bits & 7) {
-        alloc += ERL_SUB_BIN_SIZE;
-    }
-    if (num_bytes <= ERL_ONHEAP_BIN_LIMIT) {
-        alloc += heap_bin_size(num_bytes);
+    if (num_bits <= ERL_ONHEAP_BITS_LIMIT) {
+        alloc += heap_bits_size(num_bits);
     } else {
-        alloc += PROC_BIN_SIZE;
+        alloc += ERL_REFC_BITS_SIZE;
     }
 
     erts_bin_offset = 0;
 
-    /* num_bits = Number of bits to build
-     * num_bytes = Number of bytes to allocate in the binary
-     * alloc = Total number of words to allocate on heap
-     * Operands: NotUsed NotUsed Dst
-     */
-    if (num_bytes <= ERL_ONHEAP_BIN_LIMIT) {
-        ErlHeapBin *hb;
+    if (num_bits <= ERL_ONHEAP_BITS_LIMIT) {
+        ErlHeapBits *hb;
 
         gc_test(c_p, reg, 0, alloc, Live);
-        hb = (ErlHeapBin *)c_p->htop;
-        c_p->htop += heap_bin_size(num_bytes);
-        hb->thing_word = header_heap_bin(num_bytes);
-        hb->size = num_bytes;
+        hb = (ErlHeapBits *)c_p->htop;
+
+        c_p->htop += heap_bits_size(num_bits);
+        hb->thing_word = header_heap_bits(num_bits);
+        hb->size = num_bits;
+
         erts_current_bin = (byte *)hb->data;
-        new_binary = make_binary(hb);
+        return make_bitstring(hb);
     } else {
-        Binary *bptr;
-        ProcBin *pb;
+        const Uint num_bytes = NBYTES(num_bits);
+        Binary *new_binary;
 
         test_bin_vheap(c_p, reg, num_bytes / sizeof(Eterm), alloc, Live);
 
-        /*
-         * Allocate the binary struct itself.
-         */
-        bptr = erts_bin_nrml_alloc(num_bytes);
-        erts_current_bin = (byte *)bptr->orig_bytes;
+        new_binary = erts_bin_nrml_alloc(num_bytes);
+        erts_current_bin = (byte *)new_binary->orig_bytes;
 
-        /*
-         * Now allocate the ProcBin on the heap.
-         */
-        pb = (ProcBin *)c_p->htop;
-        c_p->htop += PROC_BIN_SIZE;
-        pb->thing_word = HEADER_PROC_BIN;
-        pb->size = num_bytes;
-        pb->next = MSO(c_p).first;
-        MSO(c_p).first = (struct erl_off_heap_header *)pb;
-        pb->val = bptr;
-        pb->bytes = (byte *)bptr->orig_bytes;
-        pb->flags = 0;
-        OH_OVERHEAD(&(MSO(c_p)), pb->size / sizeof(Eterm));
-        new_binary = make_binary(pb);
+        return erts_wrap_refc_bitstring(&MSO(c_p).first,
+                                        &MSO(c_p).overhead,
+                                        &HEAP_TOP(c_p),
+                                        new_binary,
+                                        erts_current_bin,
+                                        0,
+                                        num_bits);
     }
-
-    if (num_bits & 7) {
-        ErlSubBin *sb;
-
-        sb = (ErlSubBin *)c_p->htop;
-        c_p->htop += ERL_SUB_BIN_SIZE;
-        sb->thing_word = HEADER_SUB_BIN;
-        sb->size = num_bytes - 1;
-        sb->bitsize = num_bits & 7;
-        sb->offs = 0;
-        sb->bitoffs = 0;
-        sb->is_writable = 0;
-        sb->orig = new_binary;
-        new_binary = make_binary(sb);
-    }
-
-    return new_binary;
 }
 
 /*
@@ -910,7 +871,7 @@ Eterm beam_jit_bs_get_integer(Process *c_p,
                               Uint flags,
                               Uint size,
                               Uint Live) {
-    ErlBinMatchBuffer *mb;
+    ErlSubBits *sb;
 
     if (size >= SMALL_BITS) {
         Uint wordsneeded;
@@ -921,8 +882,8 @@ Eterm beam_jit_bs_get_integer(Process *c_p,
          *
          * Remember to re-acquire the matchbuffer after gc.
          */
-        mb = ms_matchbuffer(context);
-        if (mb->size - mb->offset < size) {
+        sb = (ErlSubBits *)bitstring_val(context);
+        if (sb->end - sb->start < size) {
             return THE_NON_VALUE;
         }
 
@@ -936,8 +897,8 @@ Eterm beam_jit_bs_get_integer(Process *c_p,
         context = reg[Live];
     }
 
-    mb = ms_matchbuffer(context);
-    return erts_bs_get_integer_2(c_p, size, flags, mb);
+    sb = (ErlSubBits *)bitstring_val(context);
+    return erts_bs_get_integer_2(c_p, size, flags, sb);
 }
 
 void beam_jit_bs_construct_fail_info(Process *c_p,
@@ -958,7 +919,7 @@ void beam_jit_bs_construct_fail_info(Process *c_p,
     Eterm value = am_undefined;
 
     switch (op) {
-    case BSC_OP_BINARY:
+    case BSC_OP_BITSTRING:
         Op = am_binary;
         break;
     case BSC_OP_FLOAT:
@@ -1021,8 +982,8 @@ void beam_jit_bs_construct_fail_info(Process *c_p,
         Info = am_unit;
         break;
     case BSC_INFO_DEPENDS:
-        ASSERT(op == BSC_OP_BINARY);
-        Info = is_binary(value) ? am_short : am_type;
+        ASSERT(op == BSC_OP_BITSTRING);
+        Info = is_bitstring(value) ? am_short : am_type;
         break;
     }
 
@@ -1041,16 +1002,37 @@ void beam_jit_bs_construct_fail_info(Process *c_p,
 }
 
 Sint beam_jit_bs_bit_size(Eterm term) {
-    if (is_binary(term)) {
+    if (is_bitstring(term)) {
+        Uint size = bitstring_size(term);
+
         ASSERT(sizeof(Uint) == 8); /* Only support 64-bit machines. */
-        Uint byte_size = binary_size(term);
-        return (Sint)((byte_size << 3) + binary_bitsize(term));
+        ASSERT(size <= ERTS_SINT_MAX);
+
+        return (Sint)size;
     }
 
     /* Signal error */
     return (Sint)-1;
 }
 
+Eterm beam_jit_int128_to_big(Process *p, Uint sign, Uint low, Uint high) {
+    Eterm *hp;
+    Uint arity;
+
+    arity = high ? 2 : 1;
+    hp = HeapFragOnlyAlloc(p, BIG_NEED_SIZE(arity));
+    if (sign) {
+        hp[0] = make_neg_bignum_header(arity);
+    } else {
+        hp[0] = make_pos_bignum_header(arity);
+    }
+    BIG_DIGIT(hp, 0) = low;
+    if (arity == 2) {
+        BIG_DIGIT(hp, 1) = high;
+    }
+    return make_big(hp);
+}
+
 ErtsMessage *beam_jit_decode_dist(Process *c_p, ErtsMessage *msgp) {
     if (!erts_proc_sig_decode_dist(c_p, ERTS_PROC_LOCK_MAIN, msgp, 0)) {
         /*
@@ -1070,11 +1052,11 @@ ErtsMessage *beam_jit_decode_dist(Process *c_p, ErtsMessage *msgp) {
 }
 
 /* Remove a (matched) message from the message queue. */
-Sint beam_jit_remove_message(Process *c_p,
-                             Sint FCALLS,
-                             Eterm *HTOP,
-                             Eterm *E,
-                             Uint32 active_code_ix) {
+Sint32 beam_jit_remove_message(Process *c_p,
+                               Sint32 FCALLS,
+                               Eterm *HTOP,
+                               Eterm *E,
+                               Uint32 active_code_ix) {
     ErtsMessage *msgp;
 
     ERTS_CHK_MBUF_SZ(c_p);
@@ -1142,7 +1124,6 @@ Sint beam_jit_remove_message(Process *c_p,
         Sint tok_label = 0;
         Sint tok_lastcnt = 0;
         Sint tok_serial = 0;
-        Sint len = erts_proc_sig_privqs_len(c_p);
 
         dtrace_proc_str(c_p, receiver_name);
         token2 = SEQ_TRACE_TOKEN(c_p);
@@ -1154,7 +1135,7 @@ Sint beam_jit_remove_message(Process *c_p,
         DTRACE6(message_receive,
                 receiver_name,
                 size_object(ERL_MESSAGE_TERM(msgp)),
-                len, /* This is NOT message queue len, but its something... */
+                c_p->sig_qs.mq_len,
                 tok_label,
                 tok_lastcnt,
                 tok_serial);
@@ -1236,8 +1217,8 @@ enum beam_jit_tmo_ret beam_jit_wait_timeout(Process *c_p,
 }
 
 void beam_jit_timeout(Process *c_p) {
-    if (IS_TRACED_FL(c_p, F_TRACE_RECEIVE)) {
-        trace_receive(c_p, am_clock_service, am_timeout, NULL);
+    if (ERTS_IS_P_TRACED_FL(c_p, F_TRACE_RECEIVE)) {
+        trace_receive(c_p, am_clock_service, am_timeout);
     }
     if (ERTS_PROC_GET_SAVED_CALLS_BUF(c_p)) {
         save_calls(c_p, &exp_timeout);
@@ -1251,12 +1232,14 @@ void beam_jit_timeout_locked(Process *c_p) {
     beam_jit_timeout(c_p);
 }
 
-void beam_jit_return_to_trace(Process *c_p) {
-    if (IS_TRACED_FL(c_p, F_TRACE_RETURN_TO)) {
+void beam_jit_return_to_trace(Process *c_p,
+                              Eterm session_weak_id,
+                              Eterm *frame) {
+    if (ERTS_IS_P_TRACED_FL(c_p, F_TRACE_RETURN_TO)) {
         ErtsCodePtr return_to_address;
         Uint *cpp;
 
-        cpp = (Uint *)c_p->stop;
+        cpp = (Uint *)frame;
         ASSERT(is_CP(cpp[0]));
 
         for (;;) {
@@ -1274,7 +1257,7 @@ void beam_jit_return_to_trace(Process *c_p) {
         }
 
         ERTS_UNREQ_PROC_MAIN_LOCK(c_p);
-        erts_trace_return_to(c_p, return_to_address);
+        erts_trace_return_to(c_p, return_to_address, session_weak_id);
         ERTS_REQ_PROC_MAIN_LOCK(c_p);
     }
 }
@@ -1297,8 +1280,8 @@ Eterm beam_jit_build_argument_list(Process *c_p, const Eterm *regs, int arity) {
 Export *beam_jit_handle_unloaded_fun(Process *c_p,
                                      Eterm *reg,
                                      int arity,
-                                     Eterm fun_thing,
-                                     ErtsCodeIndex code_ix) {
+                                     Eterm fun_thing) {
+    ErtsCodeIndex code_ix = erts_active_code_ix();
     Eterm module, args;
     ErlFunThing *funp;
     ErlFunEntry *fe;
@@ -1353,3 +1336,58 @@ Export *beam_jit_handle_unloaded_fun(Process *c_p,
 
     return ep;
 }
+
+bool beam_jit_is_list_of_immediates(Eterm term) {
+    while (is_list(term)) {
+        Eterm *cons = list_val(term);
+        if (!is_immed(CAR(cons))) {
+            return false;
+        }
+        term = CDR(cons);
+    }
+    return is_nil(term);
+}
+
+bool beam_jit_is_shallow_boxed(Eterm term) {
+    if (is_tuple(term)) {
+        Eterm *tuple_ptr = tuple_val(term);
+        for (unsigned i = 1; i <= arityval(*tuple_ptr); i++) {
+            if (!is_immed(tuple_ptr[i])) {
+                return false;
+            }
+        }
+        return true;
+    } else if (is_big(term)) {
+        return true;
+    } else if (is_float(term)) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+#ifdef DEBUG
+void beam_jit_invalid_heap_ptr(Process *p, Eterm term) {
+    ASSERT((void *)term <= (void *)p->heap || (void *)term >= (void *)p->hend);
+
+    erts_fprintf(stderr,
+                 "term:       %p\n"
+                 "c_p:        %p\n"
+                 "heap:       %p\n"
+                 "high_water: %p\n"
+                 "hend:       %p\n"
+                 "abandoned:  %p\n",
+                 (void *)term,
+                 p,
+                 p->heap,
+                 p->high_water,
+                 p->hend,
+                 p->abandoned_heap);
+    if (p->old_heap != NULL && p->old_hend != NULL &&
+        (void *)term < (void *)p->old_hend &&
+        (void *)term >= (void *)p->old_heap) {
+        erts_fprintf(stderr, "the term is on the old heap\n");
+    }
+    abort();
+}
+#endif
