@@ -131,34 +131,31 @@ static void beamasm_init_late_gdb() {
     entry->symfile_size = sizeof(struct debug_info);
 
     /* Insert into linked list */
-    entry->next_entry = __jit_debug_descriptor.first_entry;
-    if (entry->next_entry) {
-        entry->next_entry->prev_entry = entry;
-    } else {
-        entry->prev_entry = nullptr;
+    erts_mtx_lock(&__jit_debug_descriptor.mutex);
+
+    if (__jit_debug_descriptor.first_entry) {
+        ASSERT(__jit_debug_descriptor.first_entry->prev_entry == nullptr);
+        __jit_debug_descriptor.first_entry->prev_entry = entry;
     }
 
-    /* Insert into linked list */
-    erts_mtx_lock(&__jit_debug_descriptor.mutex);
+    entry->prev_entry = nullptr;
     entry->next_entry = __jit_debug_descriptor.first_entry;
-    if (entry->next_entry) {
-        entry->next_entry->prev_entry = entry;
-    } else {
-        entry->prev_entry = nullptr;
-    }
+    __jit_debug_descriptor.first_entry = entry;
 
     /* Register with gdb */
+    ASSERT(__jit_debug_descriptor.relevant_entry == nullptr);
     __jit_debug_descriptor.action_flag = JIT_REGISTER_FN;
-    __jit_debug_descriptor.first_entry = entry;
     __jit_debug_descriptor.relevant_entry = entry;
     __jit_debug_register_code();
+    __jit_debug_descriptor.relevant_entry = nullptr;
+
     erts_mtx_unlock(&__jit_debug_descriptor.mutex);
 }
 
-static void beamasm_update_gdb_info(std::string module_name,
-                                    ErtsCodePtr base_address,
-                                    size_t code_size,
-                                    const std::vector<AsmRange> &ranges) {
+static void *beamasm_insert_gdb_info(std::string module_name,
+                                     ErtsCodePtr base_address,
+                                     size_t code_size,
+                                     const std::vector<AsmRange> &ranges) {
     Sint symfile_size = sizeof(struct debug_info) + module_name.size() + 1;
 
     for (const auto &range : ranges) {
@@ -221,21 +218,58 @@ static void beamasm_update_gdb_info(std::string module_name,
 
     /* Insert into linked list */
     erts_mtx_lock(&__jit_debug_descriptor.mutex);
-    entry->next_entry = __jit_debug_descriptor.first_entry;
-    if (entry->next_entry) {
-        entry->next_entry->prev_entry = entry;
-    } else {
-        entry->prev_entry = nullptr;
+
+    if (__jit_debug_descriptor.first_entry) {
+        ASSERT(__jit_debug_descriptor.first_entry->prev_entry == nullptr);
+        __jit_debug_descriptor.first_entry->prev_entry = entry;
     }
 
-    /* register with gdb */
-    __jit_debug_descriptor.action_flag = JIT_REGISTER_FN;
+    entry->prev_entry = nullptr;
+    entry->next_entry = __jit_debug_descriptor.first_entry;
     __jit_debug_descriptor.first_entry = entry;
+
+    /* register with gdb */
+    ASSERT(__jit_debug_descriptor.relevant_entry == nullptr);
+    __jit_debug_descriptor.action_flag = JIT_REGISTER_FN;
     __jit_debug_descriptor.relevant_entry = entry;
     __jit_debug_register_code();
+    __jit_debug_descriptor.relevant_entry = nullptr;
+
     erts_mtx_unlock(&__jit_debug_descriptor.mutex);
+
+    /* Return a reference to our debugger entry so we can remove it when
+     * unloading. */
+    return entry;
 }
 
+static void beamasm_delete_gdb_info(void *metadata) {
+    jit_code_entry *entry = (jit_code_entry *)metadata;
+
+    erts_mtx_lock(&__jit_debug_descriptor.mutex);
+
+    /* Unlink current module from the entry list. */
+    if (entry->prev_entry) {
+        entry->prev_entry->next_entry = entry->next_entry;
+    } else {
+        __jit_debug_descriptor.first_entry = entry->next_entry;
+    }
+
+    if (entry->next_entry) {
+        entry->next_entry->prev_entry = entry->prev_entry;
+    }
+
+    /* unregister with gdb  */
+    ASSERT(__jit_debug_descriptor.relevant_entry == nullptr);
+    __jit_debug_descriptor.action_flag = JIT_UNREGISTER_FN;
+    __jit_debug_descriptor.relevant_entry = entry;
+    __jit_debug_register_code();
+    __jit_debug_descriptor.relevant_entry = nullptr;
+
+    erts_mtx_unlock(&__jit_debug_descriptor.mutex);
+
+    free((void *)entry->symfile_addr);
+    free(entry);
+}
 #endif /* HAVE_GDB_SUPPORT */
 
 #ifdef HAVE_LINUX_PERF_SUPPORT
@@ -525,15 +559,26 @@ void beamasm_metadata_late_init() {
 #endif
 }
 
-void beamasm_metadata_update(std::string module_name,
-                             ErtsCodePtr base_address,
-                             size_t code_size,
-                             const std::vector<AsmRange> &ranges) {
+void *beamasm_metadata_insert(std::string module_name,
+                              ErtsCodePtr base_address,
+                              size_t code_size,
+                              const std::vector<AsmRange> &ranges) {
 #ifdef HAVE_LINUX_PERF_SUPPORT
     perf.update(ranges);
 #endif
 
 #ifdef HAVE_GDB_SUPPORT
-    beamasm_update_gdb_info(module_name, base_address, code_size, ranges);
+    return beamasm_insert_gdb_info(module_name,
+                                   base_address,
+                                   code_size,
+                                   ranges);
+#else
+    return NULL;
+#endif
+}
+
+void beamasm_unregister_metadata(void *metadata) {
+#ifdef HAVE_GDB_SUPPORT
+    beamasm_delete_gdb_info(metadata);
 #endif
 }
