@@ -21,8 +21,6 @@
 #ifndef __SYS_H__
 #define __SYS_H__
 
-#define ERTS_SUPPORT_OLD_RECV_MARK_INSTRS
-
 #if !defined(__GNUC__) || defined(__e2k__)
 #  define ERTS_AT_LEAST_GCC_VSN__(MAJ, MIN, PL) 0
 #elif !defined(__GNUC_MINOR__)
@@ -428,6 +426,7 @@ typedef long long          Sint  erts_align_attribute(sizeof(long long));
 typedef Uint UWord;
 typedef Sint SWord;
 #define ERTS_UINT_MAX ERTS_UWORD_MAX
+#define ERTS_SINT_MAX ERTS_SWORD_MAX
 
 typedef const void *ErtsCodePtr;
 typedef UWord BeamInstr;
@@ -650,6 +649,8 @@ __decl_noreturn void __noreturn erts_exit(int n, const char*, ...);
 
 UWord erts_sys_get_page_size(void);
 
+UWord erts_sys_get_large_page_size(void);
+
 /* Size of misc memory allocated from system dependent code */
 Uint erts_sys_misc_mem_sz(void);
 
@@ -709,7 +710,6 @@ typedef struct preload {
  */
 typedef Eterm ErtsTracer;
 
-
 /*
  * This structure contains the rb tree for the erlang osenv copy
  * see erl_osenv.h for more details.
@@ -911,6 +911,9 @@ typedef struct {
 
 void sys_alloc_stat(SysAllocStat *);
 
+extern UWord sys_page_size;
+extern UWord sys_large_page_size;
+
 #if defined(DEBUG) || defined(ERTS_ENABLE_LOCK_CHECK)
 #undef ERTS_REFC_DEBUG
 #define ERTS_REFC_DEBUG
