@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2018-2025. All Rights Reserved.
+ * Copyright Ericsson AB 2018-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -177,28 +177,6 @@ typedef int BOOLEAN_T;
 #define NUM(Array) (sizeof(Array) / sizeof(*(Array)))
 
 
-/* ********************************************************************* *
- *                              SOCKET and HANDLE                        *
- * ********************************************************************* *
- */
-
-#if defined(__WIN32__)
-
-#define INVALID_EVENT NULL
-#define SOCKET_FORMAT_STR "%lld"
-
-#else
-
-#define INVALID_HANDLE (-1)
-typedef int HANDLE;
-#define INVALID_SOCKET (-1)
-typedef int SOCKET; /* A subset of HANDLE */
-#define INVALID_EVENT INVALID_HANDLE
-#define SOCKET_FORMAT_STR "%d"
-
-#endif
-
-
 #ifdef HAVE_SOCKLEN_T
 #  define SOCKLEN_T socklen_t
 #else
@@ -303,7 +281,6 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(default_send_params);      \
     GLOBAL_ATOM_DEF(delayed_ack_time);         \
     GLOBAL_ATOM_DEF(dgram);                    \
-    GLOBAL_ATOM_DEF(disabled);                 \
     GLOBAL_ATOM_DEF(disable_fragments);        \
     GLOBAL_ATOM_DEF(dlci);                     \
     GLOBAL_ATOM_DEF(dma);                      \
@@ -322,7 +299,6 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(eether);                   \
     GLOBAL_ATOM_DEF(efile);                    \
     GLOBAL_ATOM_DEF(egp);                      \
-    GLOBAL_ATOM_DEF(enabled);                  \
     GLOBAL_ATOM_DEF(enotsup);                  \
     GLOBAL_ATOM_DEF(eor);                      \
     GLOBAL_ATOM_DEF(error);                    \
@@ -542,6 +518,7 @@ typedef long ssize_t;
     GLOBAL_ATOM_DEF(sendmsg);                  \
     GLOBAL_ATOM_DEF(sendsrcaddr);              \
     GLOBAL_ATOM_DEF(sendto);                   \
+    GLOBAL_ATOM_DEF(sendv);                    \
     GLOBAL_ATOM_DEF(seqpacket);                \
     GLOBAL_ATOM_DEF(setfib);                   \
     GLOBAL_ATOM_DEF(set_peer_primary_addr);    \
@@ -663,7 +640,6 @@ GLOBAL_ERROR_REASON_ATOM_DEFS;
 #define MKUL(E,UL)          enif_make_ulong((E), (UL))
 
 #define MCREATE(N)          enif_mutex_create((N))
-#define MCREATE3(P, B, S)   esock_mutex_create((P), (B), (S))
 #define MDESTROY(M)         enif_mutex_destroy((M))
 #define MLOCK(M)            enif_mutex_lock((M))
 #define MUNLOCK(M)          enif_mutex_unlock((M))
@@ -681,24 +657,27 @@ GLOBAL_ERROR_REASON_ATOM_DEFS;
 #define IS_UNDEFINED(T)      IS_IDENTICAL((T), esock_atom_undefined)
 #define IS_OK(T)             IS_IDENTICAL((T), esock_atom_ok)
 
-#define IS_ATOM(E,    TE) enif_is_atom((E),   (TE))
-#define IS_BIN(E,     TE) enif_is_binary((E), (TE))
-#define IS_LIST(E,    TE) enif_is_list((E),   (TE))
-#define IS_MAP(E,     TE) enif_is_map((E), (TE))
-#define IS_NUM(E,     TE) enif_is_number((E), (TE))
-#define IS_TUPLE(E,   TE) enif_is_tuple((E),  (TE))
-#define IS_INTEGER(E, TE) esock_is_integer((E),   (TE))
+#define IS_ATOM(E,    TE)    enif_is_atom((E),   (TE))
+#define IS_BIN(E,     TE)    enif_is_binary((E), (TE))
+#define IS_LIST(E,    TE)    enif_is_list((E),   (TE))
+#define IS_MAP(E,     TE)    enif_is_map((E), (TE))
+#define IS_NUM(E,     TE)    enif_is_number((E), (TE))
+#define IS_TUPLE(E,   TE)    enif_is_tuple((E),  (TE))
+#define IS_INTEGER(E, TE)    esock_is_integer((E),   (TE))
 
-#define IS_PID_UNDEF(P)   enif_is_pid_undefined((P))
-#define SET_PID_UNDEF(P)  enif_set_pid_undefined((P))
+#define IS_PID_UNDEF(P)      enif_is_pid_undefined((P))
+#define SET_PID_UNDEF(P)     enif_set_pid_undefined((P))
 
 #define GET_ATOM_LEN(E, TE, LP) \
     enif_get_atom_length((E), (TE), (LP), ERL_NIF_LATIN1)
 #define GET_ATOM(E, TE, BP, MAX) \
     enif_get_atom((E), (TE), (BP), (MAX), ERL_NIF_LATIN1)
-#define GET_BIN(E, TE, BP)          enif_inspect_iolist_as_binary((E), (TE), (BP))
+#define GET_BIN(E, TE, BP)          \
+    enif_inspect_iolist_as_binary((E), (TE), (BP))
 #define GET_INT(E, TE, IP)          enif_get_int((E), (TE), (IP))
 #define GET_INT64(E, TE, IP)        enif_get_int64((E), (TE), (IP))
+#define GET_IOV(ME, EIOV, T, IOV)                       \
+    enif_inspect_iovec(NULL, (ME), (EIOV), (T), (IOV))
 #define GET_LIST_ELEM(E, L, HP, TP) enif_get_list_cell((E), (L), (HP), (TP))
 #define GET_LIST_LEN(E, L, LP)      enif_get_list_length((E), (L), (LP))
 #define GET_LONG(E, TE, LP)         enif_get_long((E), (TE), (LP))
@@ -712,7 +691,7 @@ GLOBAL_ERROR_REASON_ATOM_DEFS;
 #define GET_MAP_VAL(E, M, K, V)     enif_get_map_value((E), (M), (K), (V))
 
 #define ALLOC_BIN(SZ, BP)         enif_alloc_binary((SZ), (BP))
-#define REALLOC_BIN(SZ, BP)       enif_realloc_binary((SZ), (BP))
+#define REALLOC_BIN(BP, SZ)       enif_realloc_binary((BP), (SZ))
 #define FREE_BIN(BP)              enif_release_binary((BP))
 
 #define FREE_IOVEC(IV)            enif_free_iovec((IV))
