@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2025. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -632,13 +632,9 @@ do_something() ->
 
 append_empty_is_same(Config) when is_list(Config) ->
     NonWritableBin = <<"123">>,
-
-    %% Should use erts_debug:same/2, but this was disabled in PR-9372
-    %% (ERIERL-1177) because the optimization was unsafe.
-    true = NonWritableBin =:= append(NonWritableBin, <<>>),
+    true = erts_debug:same(NonWritableBin, append(NonWritableBin, <<>>)),
     WritableBin = <<(id(<<>>))/binary,0,1,2,3,4,5,6,7>>,
-    true = WritableBin =:= append(WritableBin, <<>>),
-
+    true = erts_debug:same(WritableBin, append(WritableBin, <<>>)),
     ok.
 
 append(A, B) ->
@@ -787,10 +783,16 @@ dynamic_3(Bef, N, {Int0,Lpad,Rpad,Dynamic}=Data, Count) ->
     Dynamic(Bef, N, Int1, Lpad, Rpad),
     Dynamic(Bef, N, -Int1, Lpad, Rpad),
 
-    %% OTP-7085: Test a small number in a wide field.
+    %% OTP-7085: Test a small number in a wide segment.
     Int2 = Int0 band 16#FFFFFF,
     Dynamic(Bef, N, Int2, Lpad, Rpad),
     Dynamic(Bef, N, -Int2, Lpad, Rpad),
+
+    %% Test a bignum in a short segment.
+    Int4 = ((Lpad bxor Rpad) bsl N) bor Int0,
+    Dynamic(Bef, N, Int4, Lpad, Rpad),
+    Dynamic(Bef, N, -Int4, Lpad, Rpad),
+
     dynamic_3(Bef, N-1, Data, Count+1).
 
 dynamic_big(Bef, N, Int, Lpad, Rpad) ->
