@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2025. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -24,10 +24,12 @@
 %%% Tests the trace BIF.
 %%%
 
--export([all/0, suite/0, init_per_testcase/2, end_per_testcase/2,
+-export([all/0, suite/0, groups/0,
+         init_per_suite/1, end_per_suite/1,
+         init_per_group/2, end_per_group/2,
+         init_per_testcase/2, end_per_testcase/2,
          link_receive_call_correlation/0,
-         receive_trace/1, receive_trace_non_fetching_receiver/1,
-         link_receive_call_correlation/1, self_send/1,
+         receive_trace/1, link_receive_call_correlation/1, self_send/1,
 	 timeout_trace/1, send_trace/1,
 	 procs_trace/1, dist_procs_trace/1, procs_new_trace/1,
 	 suspend/1, suspend_exit/1, suspender_exit/1,
@@ -38,22 +40,34 @@
 	 system_monitor_args/1, more_system_monitor_args/1,
 	 system_monitor_long_gc_1/1, system_monitor_long_gc_2/1, 
 	 system_monitor_large_heap_1/1, system_monitor_large_heap_2/1,
-	 system_monitor_long_schedule/1,
+	 system_monitor_long_schedule/1, system_monitor_long_message_queue/1,
+         system_monitor_long_message_queue_ignore/1,
 	 bad_flag/1, trace_delivered/1, trap_exit_self_receive/1,
          trace_info_badarg/1, erl_704/1, ms_excessive_nesting/1]).
 
+-nifs([slow_nif/0]).
+
 -include_lib("common_test/include/ct.hrl").
 
 %%% Internal exports
 -export([process/1]).
 
+-undef(line).
+-define(line, erlang:display(?LINE)).
+
 suite() ->
     [{ct_hooks,[ts_install_cth]},
      {timetrap, {minutes, 1}}].
 
-all() -> 
-    [cpu_timestamp, receive_trace, receive_trace_non_fetching_receiver,
-     link_receive_call_correlation, self_send, timeout_trace,
+all() ->
+    trace_sessions:all().
+
+groups() ->
+    trace_sessions:groups(testcases()).
+
+testcases() ->
+    [cpu_timestamp, receive_trace, link_receive_call_correlation,
+     self_send, timeout_trace,
      send_trace, procs_trace, dist_procs_trace, suspend,
      suspend_exit, suspender_exit,
      suspend_system_limit, suspend_opts, suspend_waiting,
@@ -63,27 +77,54 @@ all() ->
      more_system_monitor_args, system_monitor_long_gc_1,
      system_monitor_long_gc_2, system_monitor_large_heap_1,
      system_monitor_long_schedule,
-     system_monitor_large_heap_2, bad_flag, trace_delivered,
+     system_monitor_large_heap_2, system_monitor_long_message_queue,
+     system_monitor_long_message_queue_ignore,
+     bad_flag, trace_delivered,
      trap_exit_self_receive, trace_info_badarg, erl_704,
      ms_excessive_nesting].
 
-init_per_testcase(_Case, Config) ->
-    [{receiver,spawn(fun receiver/0)}|Config].
+init_per_suite(Config) ->
+    trace_sessions:init_per_suite(Config).
+
+end_per_suite(Config) ->
+    trace_sessions:end_per_suite(Config).
+
+
+init_per_group(Group, Config) ->
+    trace_sessions:init_per_group(Group, Config).
+
+end_per_group(Group, Config) ->
+    trace_sessions:end_per_group(Group, Config).
+
+init_per_testcase(_Case, Config0) ->
+    Config1 = [{receiver,spawn(fun receiver/0)}|Config0],
+    trace_sessions:init_per_testcase(Config1).
 
 end_per_testcase(_Case, Config) ->
     Receiver = proplists:get_value(receiver, Config),
     unlink(Receiver),
     exit(Receiver, die),
 
-    erts_test_utils:ept_check_leaked_nodes(Config).
+    (erts_test_utils:ept_check_leaked_nodes(Config)
+     andalso
+     trace_sessions:end_per_testcase(Config)).
+
+erlang_trace(A,B,C) ->
+    trace_sessions:erlang_trace(A,B,C).
+
+erlang_trace_pattern(A,B,C) ->
+    trace_sessions:erlang_trace_pattern(A,B,C).
+
+erlang_trace_info(A,B) ->
+    trace_sessions:erlang_trace_info(A,B).
 
 %% No longer testing anything, just reporting whether cpu_timestamp
 %% is enabled or not.
 cpu_timestamp(Config) when is_list(Config) ->
     %% Test whether cpu_timestamp is implemented on this platform.
-    Works = try erlang:trace(all, true, [cpu_timestamp]) of
+    Works = try erlang_trace(all, true, [cpu_timestamp]) of
                 _ ->
-                    erlang:trace(all, false, [cpu_timestamp]),
+                    erlang_trace(all, false, [cpu_timestamp]),
                     true
             catch
                 error:badarg -> false
@@ -100,7 +141,7 @@ receive_trace(Config) when is_list(Config) ->
     Receiver = proplists:get_value(receiver, Config),
 
     %% Trace the process; make sure that we receive the trace messages.
-    1 = erlang:trace(Receiver, true, ['receive']),
+    1 = erlang_trace(Receiver, true, ['receive']),
     Hello = {hello, world},
     Receiver ! Hello,
     {trace, Receiver, 'receive', Hello} = receive_first_trace(),
@@ -186,24 +227,24 @@ receive_trace(Config) when is_list(Config) ->
     Receiver ! {set_timeout, 10},
     {trace, Receiver, 'receive', {set_timeout, 10}} = receive_first_trace(),
     {trace, Receiver, 'receive', timeout} = receive_first_trace(),
-    erlang:trace_pattern('receive', [{[clock_service,undefined,timeout], [], []}], []),
+    erlang_trace_pattern('receive', [{[clock_service,undefined,timeout], [], []}], []),
     Receiver ! {set_timeout, 7},
     {trace, Receiver, 'receive', timeout} = receive_first_trace(),
-    erlang:trace_pattern('receive', true, []),
+    erlang_trace_pattern('receive', true, []),
 
     %% Another process should not be able to trace Receiver.
     process_flag(trap_exit, true),
-    Intruder = fun_spawn(fun() -> erlang:trace(Receiver, true, ['receive']) end),
+    Intruder = fun_spawn(fun() -> erlang_trace(Receiver, true, ['receive']) end),
     {'EXIT', Intruder, {badarg, _}} = receive_first(),
 
     %% Untrace the process; we should not receive anything.
-    1 = erlang:trace(Receiver, false, ['receive']),
+    1 = erlang_trace(Receiver, false, ['receive']),
     Receiver ! {hello, there},
     Receiver ! any_garbage,
     receive_nothing(),
 
     %% Verify restrictions in matchspec for 'receive'
-    F3 = fun (Pat) -> {'EXIT', {badarg,_}} = (catch erlang:trace_pattern('receive', Pat, [])) end,
+    F3 = fun (Pat) -> {'EXIT', {badarg,_}} = (catch erlang_trace_pattern('receive', Pat, [])) end,
     WC = ['_','_','_'],
     F3([{WC,[],[{message, {process_dump}}]}]),
     F3([{WC,[{is_seq_trace}],[]}]),
@@ -220,45 +261,6 @@ receive_trace(Config) when is_list(Config) ->
 
     ok.
 
-receive_trace_non_fetching_receiver(Config) when is_list(Config) ->
-    receive_trace_non_fetching_receiver_test(node()),
-    {ok, Peer, Node} = ?CT_PEER(),
-    receive_trace_non_fetching_receiver_test(Node),
-    peer:stop(Peer),
-    ok.
-
-receive_trace_non_fetching_receiver_test(Node) ->
-    Msgs = lists:map(fun (N) -> {msg, N} end, lists:seq(1, 20)),
-
-    Receiver = spawn_link(fun () -> receive after infinity -> ok end end),
-    Sender = spawn_link(Node, fun Sender() ->
-                                      receive Msg -> Receiver ! Msg end,
-                                      Sender()
-                              end),
-
-    1 = erlang:trace(Receiver, true, ['receive']),
-
-    lists:foreach(fun (Msg) ->
-                          Tmo = rand:uniform(101) - 1,
-                          receive after Tmo -> ok end,
-                          Sender ! Msg
-                  end, Msgs),
-
-    lists:foreach(fun (Msg) ->
-                          {trace, Receiver, 'receive', Msg}
-                              = receive_first_trace()
-                        end, Msgs),
-
-    {messages, Msgs} = process_info(Receiver, messages),
-
-    TrapExit = process_flag(trap_exit, true),
-    exit(Receiver, kill),
-    exit(Sender, kill),
-    receive {'EXIT', Receiver, _} -> ok end,
-    receive {'EXIT', Sender, _} -> ok end,
-    _ = process_flag(trap_exit, TrapExit),
-    ok.
-
 %% Tests that receive of a message always happens before a call with
 %% that message and that links/unlinks are ordered together with the
 %% 'receive'.
@@ -274,8 +276,8 @@ link_receive_call_correlation(Config) when is_list(Config) ->
     process_flag(trap_exit, true),
 
     %% Trace the process; make sure that we receive the trace messages.
-    1 = erlang:trace(Receiver, true, ['receive', procs, call, timestamp, scheduler_id]),
-    1 = erlang:trace_pattern({?MODULE, receive_msg, '_'}, [], [local]),
+    1 = erlang_trace(Receiver, true, ['receive', procs, call, timestamp, scheduler_id]),
+    1 = erlang_trace_pattern({?MODULE, receive_msg, '_'}, [], [local]),
 
     Num = 100,
 
@@ -422,7 +424,7 @@ self_send(Config) when is_list(Config) ->
     end,
     Self = self(),
     SelfSender = fun_spawn(Fun, [Fun, Self]),
-    erlang:trace(SelfSender, true, ['receive', 'send']),
+    erlang_trace(SelfSender, true, ['receive', 'send']),
     SelfSender ! go_ahead,
     receive {trace, SelfSender, 'receive', go_ahead} -> ok end,
     receive {trace, SelfSender, 'receive', from_myself} -> ok end,
@@ -436,7 +438,7 @@ self_send(Config) when is_list(Config) ->
 %% Test that we can receive timeout traces.
 timeout_trace(Config) when is_list(Config) ->
     Process = fun_spawn(fun process/0),
-    1 = erlang:trace(Process, true, ['receive']),
+    1 = erlang_trace(Process, true, ['receive']),
     Process ! timeout_please,
     {trace, Process, 'receive', timeout_please} = receive_first_trace(),
     {trace, Process, 'receive', timeout} = receive_first_trace(),
@@ -451,7 +453,7 @@ send_trace(Config) when is_list(Config) ->
     Receiver = proplists:get_value(receiver, Config),
 
     %% Check that a message sent to another process is traced.
-    1 = erlang:trace(Sender, true, [send]),
+    1 = erlang_trace(Sender, true, [send]),
     F1 = fun (Pat) ->
 		 set_trace_pattern(send, Pat, []),
 		 Sender ! {send_please, Receiver, to_receiver},
@@ -496,8 +498,8 @@ send_trace(Config) when is_list(Config) ->
 		       false,
 		       [{['_','_'],[],[{message,false}]}],
 		       [{['_','_'],[],[{silent,true}]}]]),
-    erlang:trace_pattern(send, true, []),
-    erlang:trace(Sender, false, [silent]),
+    erlang_trace_pattern(send, true, []),
+    erlang_trace(Sender, false, [silent]),
 
     %% Check that a message sent to another registered process is traced.
     register(?MODULE,Receiver),
@@ -527,8 +529,8 @@ send_trace(Config) when is_list(Config) ->
 		       [{['_','_'],[],[{silent,true}]}]
 		      ]),
     unregister(?MODULE),
-    erlang:trace_pattern(send, true, []),
-    erlang:trace(Sender, false, [silent]),
+    erlang_trace_pattern(send, true, []),
+    erlang_trace(Sender, false, [silent]),
 
     %% Check that a message sent to this process is traced.
     F5 = fun (Pat) ->
@@ -560,41 +562,41 @@ send_trace(Config) when is_list(Config) ->
 
     %% Check that a message sent to unknown registrated process is traced.
     BadargSender = fun_spawn(fun sender/0),
-    1 = erlang:trace(BadargSender, true, [send]),
+    1 = erlang_trace(BadargSender, true, [send]),
     unlink(BadargSender),
     BadargSender ! {send_please, not_registered, to_unknown},
     {trace, BadargSender, send, to_unknown, not_registered} = receive_first_trace(),
     receive_nothing(),
 
     %% Another process should not be able to trace Sender.
-    Intruder = fun_spawn(fun() -> erlang:trace(Sender, true, [send]) end),
+    Intruder = fun_spawn(fun() -> erlang_trace(Sender, true, [send]) end),
     {'EXIT', Intruder, {badarg, _}} = receive_first(),
 
     %% Untrace the sender process and make sure that we receive no more
     %% trace messages.
-    1 = erlang:trace(Sender, false, [send]),
+    1 = erlang_trace(Sender, false, [send]),
     Sender ! {send_please, Receiver, to_receiver},
     Sender ! {send_please, self(), to_myself_again},
     receive to_myself_again -> ok end,
     receive_nothing(),
     
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, true, [global])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, true, [local])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, true, [meta])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, true, [{meta,self()}])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, true, [call_count])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, true, [call_time])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, restart, [])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, pause, [])),
-    {'EXIT',{badarg,_}} = (catch erlang:trace_pattern(send, [{['_','_'],[],[{caller}]}], [])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, true, [global])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, true, [local])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, true, [meta])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, true, [{meta,self()}])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, true, [call_count])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, true, [call_time])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, restart, [])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, pause, [])),
+    {'EXIT',{badarg,_}} = (catch erlang_trace_pattern(send, [{['_','_'],[],[{caller}]}], [])),
 
     %% Done.
     ok.
 
 set_trace_pattern(_, no, _) -> 0;
 set_trace_pattern(MFA, Pat, Flg) ->
-    R = erlang:trace_pattern(MFA, Pat, Flg),
-    {match_spec, Pat} = erlang:trace_info(MFA, match_spec),
+    R = erlang_trace_pattern(MFA, Pat, Flg),
+    {match_spec, Pat} = erlang_trace_info(MFA, match_spec),
     R.
 
 %% Test trace(Pid, How, [procs]).
@@ -608,7 +610,7 @@ procs_trace(Config) when is_list(Config) ->
     Proc2 = spawn(?MODULE, process, [Self]),
     io:format("Proc2 = ~p ~n", [Proc2]),
     %%
-    1 = erlang:trace(Proc1, true, [procs, set_on_first_spawn]),
+    1 = erlang_trace(Proc1, true, [procs, set_on_first_spawn]),
     MFA = {?MODULE, process, [Self]},
     %%
     %% spawn, link
@@ -677,7 +679,7 @@ procs_trace(Config) when is_list(Config) ->
     receive_nothing(),
     %%
     %% exit (not linked to tracing process)
-    1 = erlang:trace(Proc2, true, [procs]),
+    1 = erlang_trace(Proc2, true, [procs]),
     Reason2 = make_ref(),
     Proc2 ! {exit_please, Reason2},
     {trace, Proc2, exit, Reason2} = receive_first_trace(),
@@ -696,7 +698,7 @@ dist_procs_trace(Config) when is_list(Config) ->
     Proc2 = spawn(OtherNode, ?MODULE, process, [Self]),
     io:format("Proc2 = ~p ~n", [Proc2]),
     %%
-    1 = erlang:trace(Proc1, true, [procs]),
+    1 = erlang_trace(Proc1, true, [procs]),
     MFA = {?MODULE, process, [Self]},
     %%
     %% getting_unlinked by exit()
@@ -757,7 +759,7 @@ procs_new_trace(Config) when is_list(Config) ->
     Proc1 = spawn_link(?MODULE, process, [Self]),
     io:format("Proc1 = ~p ~n", [Proc1]),
     %%
-    0 = erlang:trace(new, true, [procs]),
+    0 = erlang_trace(new, true, [procs]),
 
     MFA = {?MODULE, process, [Self]},
     %%
@@ -788,7 +790,7 @@ set_on_spawn(Config) when is_list(Config) ->
     %% Create and trace a process with the set_on_spawn flag.
     %% Make sure it is traced.
     Father_SOS = fun_spawn(fun process/0),
-    1 = erlang:trace(Father_SOS, true, [send, set_on_spawn]),
+    1 = erlang_trace(Father_SOS, true, [send, set_on_spawn]),
     true = is_send_traced(Father_SOS, Listener, sos_father),
 
     %% Have the process spawn of two children and test that they
@@ -812,7 +814,7 @@ set_on_first_spawn(Config) when is_list(Config) ->
     %% Create and trace a process with the set_on_first_spawn flag.
     %% Make sure it is traced.
     Parent = fun_spawn(fun process/0),
-    1 = erlang:trace(Parent, true, [send, set_on_first_spawn]),
+    1 = erlang_trace(Parent, true, [send, set_on_first_spawn]),
     is_send_traced(Parent, Listener, sos_father),
 
     %% Have the process spawn off three children and test that the
@@ -832,7 +834,7 @@ set_on_link(_Config) ->
     %% Create and trace a process with the set_on_link flag.
     %% Make sure it is traced.
     Father_SOL = fun_spawn(fun process/0),
-    1 = erlang:trace(Father_SOL, true, [send, set_on_link]),
+    1 = erlang_trace(Father_SOL, true, [send, set_on_link]),
     true = is_send_traced(Father_SOL, Listener, sol_father),
 
     %% Have the process spawn of two children and test that they
@@ -856,7 +858,7 @@ set_on_first_link(_Config) ->
     %% Create and trace a process with the set_on_first_spawn flag.
     %% Make sure it is traced.
     Parent = fun_spawn(fun process/0),
-    1 = erlang:trace(Parent, true, [send, set_on_first_link]),
+    1 = erlang_trace(Parent, true, [send, set_on_first_link]),
     is_send_traced(Parent, Listener, sol_father),
 
     %% Have the process spawn off three children and test that the
@@ -889,13 +891,16 @@ system_monitor_args(Config) when is_list(Config) ->
            end,
     {Self,[{large_heap,MinN}]} = erlang:system_monitor(),
     {Self,[{large_heap,MinN}]} = 
-    erlang:system_monitor(Self, [busy_port]),
+        erlang:system_monitor(Self,[{long_message_queue, {100,101}}]),
+    {Self,[{long_message_queue,{100,101}}]} = erlang:system_monitor(),
+    {Self,[{long_message_queue,{100,101}}]} =
+        erlang:system_monitor(Self, [busy_port]),
     {Self,[busy_port]} = erlang:system_monitor(),
     {Self,[busy_port]} = 
     erlang:system_monitor({Self,[busy_dist_port]}),
     {Self,[busy_dist_port]} = erlang:system_monitor(),
     All = lists:sort([busy_port,busy_dist_port,
-                      {long_gc,1},{large_heap,65535}]),
+                      {long_gc,1},{large_heap,65535},{long_message_queue,{99,100}}]),
     {Self,[busy_dist_port]} = erlang:system_monitor(Self, All),
     {Self,A1} = erlang:system_monitor(),
     All = lists:sort(A1),
@@ -929,6 +934,14 @@ system_monitor_args(Config) when is_list(Config) ->
     (catch erlang:system_monitor(Self,[{large_heap,-1}])),
     {'EXIT',{badarg,_}} = 
     (catch erlang:system_monitor({Self,[{large_heap,atom}]})),
+    {'EXIT',{badarg,_}} = 
+        (catch erlang:system_monitor(Self,[{long_message_queue, {100,100}}])),
+    {'EXIT',{badarg,_}} = 
+        (catch erlang:system_monitor(Self,[{long_message_queue, {-1,1}}])),
+    {'EXIT',{badarg,_}} = 
+        (catch erlang:system_monitor(Self,[{long_message_queue, {0,-1}}])),
+    {'EXIT',{badarg,_}} = 
+        (catch erlang:system_monitor(Self,[{long_message_queue, {-1,0}}])),
     ok.
 
 
@@ -981,6 +994,10 @@ start_monitor() ->
 %% Tests erlang:system_monitor(Pid, [{long_schedule,Time}])
 system_monitor_long_schedule(Config) when is_list(Config) ->
     Path = proplists:get_value(data_dir, Config),
+    case erlang:load_nif(filename:join(Path,"trace_SUITE"), []) of
+        ok -> ok;
+        {error, {reload,_}} -> ok
+    end,
     erl_ddll:start(),
     case (catch load_driver(Path, slow_drv)) of
         ok ->
@@ -988,10 +1005,14 @@ system_monitor_long_schedule(Config) when is_list(Config) ->
         _Error ->
             {skip, "Unable to load slow_drv (windows or no usleep()?)"}
     end.
+
+slow_nif() ->
+    erlang:nif_error("NIF not loaded").
+
 do_system_monitor_long_schedule() ->
     start_monitor(),
-    Port = open_port({spawn_driver,slow_drv}, []),
-    "ok" = erlang:port_control(Port,0,[]),
+
+    slow_nif(),
     Self = self(),
     receive
         {Self,L} when is_list(L) ->
@@ -999,6 +1020,8 @@ do_system_monitor_long_schedule() ->
     after 1000 ->
             ct:fail(no_trace_of_pid)
     end,
+
+    Port = open_port({spawn_driver,slow_drv}, []),
     "ok" = erlang:port_control(Port,1,[]),
     receive
         {Port,LL} when is_list(LL) ->
@@ -1209,6 +1232,93 @@ large_heap_check(Pid, Size, Result) ->
               Result
     end.
 
+
+system_monitor_long_message_queue(Config) when is_list(Config) ->
+    Self = self(),
+    SMonPrxy = spawn_link(fun () -> smon_lmq_proxy(Self) end),
+    erlang:system_monitor(SMonPrxy,[{long_message_queue, {50,100}}]),
+    erlang:yield(),
+    lists:foreach(fun (_) -> self() ! hello end, lists:seq(1, 100)),
+    receive {monitor,Self,long_message_queue,true} -> ok
+    after 5000 -> ct:fail(missing_on_message)
+    end,
+
+    lists:foreach(fun (_) -> self() ! hello end, lists:seq(1, 10)),
+    receive {monitor,Self,long_message_queue,_} = Msg0 -> ct:fail({unexpected_message, Msg0})
+    after 1000 -> ok
+    end,
+
+    lists:foreach(fun (_) -> receive hello -> ok end end, lists:seq(1, 50)),
+    receive {monitor,Self,long_message_queue,_} = Msg1 -> ct:fail({unexpected_message, Msg1})
+    after 1000 -> ok
+    end,
+
+    lists:foreach(fun (_) -> self() ! hello end, lists:seq(1, 50)),
+    receive {monitor,Self,long_message_queue,_} = Msg2 -> ct:fail({unexpected_message, Msg2})
+    after 1000 -> ok
+    end,
+
+    lists:foreach(fun (_) -> receive hello -> ok end end, lists:seq(1, 60)),
+    receive {monitor,Self,long_message_queue,false} -> ok
+    after 5000 -> ct:fail(missing_off_message)
+    end,
+
+    lists:foreach(fun (_) -> self() ! hello end, lists:seq(1, 100)),
+    receive {monitor,Self,long_message_queue,true} -> ok
+    after 5000 -> ct:fail(missing_on_message)
+    end,
+
+    lists:foreach(fun (_) -> receive hello -> ok end end, lists:seq(1, 100)),
+    receive {monitor,Self,long_message_queue,false} -> ok
+    after 5000 -> ct:fail(missing_off_message)
+    end,
+
+    lists:foreach(fun (_) -> receive hello -> ok end end, lists:seq(1, 50)),
+    {message_queue_len, 0} = process_info(self(), message_queue_len),
+
+    unlink(SMonPrxy),
+    exit(SMonPrxy, kill),
+    false = is_process_alive(SMonPrxy),
+
+    erlang:system_monitor(undefined),
+    ok.
+
+smon_lmq_proxy(To) ->
+    receive Msg -> To ! Msg end,
+    smon_lmq_proxy(To).
+
+system_monitor_long_message_queue_ignore(Config) when is_list(Config) ->
+    %%
+    %% Ensure that messages are delivered and monitored even if a
+    %% process ignores the message queue while continuesly executing.
+    %%
+    erlang:system_monitor(self(),[{long_message_queue, {50,100}}]),
+    Pid = spawn_opt(fun ignore_messages_working/0, [{priority,low}, link]),
+    lists:foreach(fun (_) -> Pid ! hello end, lists:seq(1, 50)),
+    receive {monitor,Pid,long_message_queue,_} = Msg0 -> ct:fail({unexpected_message, Msg0})
+    after 1000 -> ok
+    end,
+
+    lists:foreach(fun (_) -> Pid ! hello end, lists:seq(1, 50)),
+    receive {monitor,Pid,long_message_queue,true} -> ok
+    after 5000 -> ct:fail(missing_on_message)
+    end,
+
+    unlink(Pid),
+    exit(Pid, kill),
+    false = is_process_alive(Pid),
+
+    erlang:system_monitor(undefined),
+
+    ok.
+
+ignore_messages_working() ->
+    _ = id(lists:seq(1, 10000)),
+    ignore_messages_working().
+
+id(X) ->
+    X.
+
 seq(N, M) ->
     seq(N, M, []).
 
@@ -1551,24 +1661,29 @@ suspend_opts(Config) when is_list(Config) ->
     SI = repeat_acc(SF, TC, #susp_info{}),
     erlang:suspend_process(Tok, [asynchronous]),
     %% Verify that it eventually suspends
-    WaitTime0 = 10,
-    WaitTime1 = case {erlang:system_info(debug_compiled),
-                      erlang:system_info(lock_checking)} of
-                    {false, false} ->
-                        WaitTime0;
-                    {false, true} ->
-                        WaitTime0*5;
-                    _ ->
-                        WaitTime0*10
-                end,
-    WaitTime = case {erlang:system_info(schedulers_online),
-                     erlang:system_info(logical_processors)} of
-                   {Schdlrs, CPUs} when is_integer(CPUs),
-                                        Schdlrs =< CPUs ->
-                       WaitTime1;
-                   _ ->
-                       WaitTime1*10
-               end,
+    WaitTime0 = 20,
+    WaitTime1 = WaitTime0 *
+        case erlang:system_info(emu_type) of
+            debug -> 2;
+            gcov -> 2;
+            asan -> 2;
+            valgrind -> 10;
+            _ -> 1
+        end,
+    WaitTime2 = WaitTime1 *
+        case erlang:system_info(lock_checking) of
+            true -> 5;
+            false -> 1
+        end,
+    WaitTime = WaitTime2 *
+        case {erlang:system_info(schedulers_online),
+              erlang:system_info(logical_processors)} of
+            {Schdlrs, CPUs} when is_integer(CPUs),
+                                 Schdlrs =< CPUs ->
+                1;
+            _ ->
+                10
+        end,
     receive after WaitTime -> ok end,
     1 = suspend_count(Tok),
     erlang:suspend_process(Tok, [asynchronous]),
@@ -1650,37 +1765,37 @@ suspend_waiting(Config) when is_list(Config) ->
     ok.
 
 
-%% Test that erlang:trace(new, true, ...) is cleared when tracer dies.
+%% Test that trace(new, true, ...) is cleared when tracer dies.
 new_clear(Config) when is_list(Config) ->
     Tracer = proplists:get_value(receiver, Config),
 
-    0 = erlang:trace(new, true, [send, {tracer, Tracer}]),
-    {flags, [send]} = erlang:trace_info(new, flags),
-    {tracer, Tracer} = erlang:trace_info(new, tracer),
+    0 = erlang_trace(new, true, [send, {tracer, Tracer}]),
+    {flags, [send]} = erlang_trace_info(new, flags),
+    {tracer, Tracer} = erlang_trace_info(new, tracer),
     Mref = erlang:monitor(process, Tracer),
     true = exit(Tracer, done),
     receive
         {'DOWN',Mref,_,_,_} -> ok
     end,
-    {flags, []} = erlang:trace_info(new, flags),
-    {tracer, []} = erlang:trace_info(new, tracer),
+    {flags, []} = erlang_trace_info(new, flags),
+    {tracer, []} = erlang_trace_info(new, tracer),
     ok.
 
 
 
-%% Test that erlang:trace(all, false, ...) works without tracer.
+%% Test that trace(all, false, ...) works without tracer.
 existing_clear(Config) when is_list(Config) ->
     Self = self(),
 
     Tracer = proplists:get_value(receiver, Config),
-    N = erlang:trace(existing, true, [send, {tracer, Tracer}]),
-    {flags, [send]} = erlang:trace_info(Self, flags),
-    {tracer, Tracer} = erlang:trace_info(Self, tracer),
-    M = erlang:trace(all, false, [all]),
+    N = erlang_trace(existing, true, [send, {tracer, Tracer}]),
+    {flags, [send]} = erlang_trace_info(Self, flags),
+    {tracer, Tracer} = erlang_trace_info(Self, tracer),
+    M = erlang_trace(all, false, [all]),
     io:format("Started trace on ~p processes and stopped on ~p~n", 
               [N, M]),
-    {flags, []} = erlang:trace_info(Self, flags),
-    {tracer, []} = erlang:trace_info(Self, tracer),
+    {flags, []} = erlang_trace_info(Self, flags),
+    {tracer, []} = erlang_trace_info(Self, tracer),
     M = N, % Used to be N + 1, but from 19.0 the tracer is also traced
 
     ok.
@@ -1692,25 +1807,25 @@ tracer_die(Config) when is_list(Config) ->
 
     Tracer = spawn_link(fun receiver/0),
     timer:sleep(1),
-    N = erlang:trace(existing, true, [send, {tracer, Tracer}]),
-    {flags, [send]} = erlang:trace_info(Proc, flags),
-    {tracer, Tracer} = erlang:trace_info(Proc, tracer),
+    N = erlang_trace(existing, true, [send, {tracer, Tracer}]),
+    {flags, [send]} = erlang_trace_info(Proc, flags),
+    {tracer, Tracer} = erlang_trace_info(Proc, tracer),
     unlink(Tracer),
     exit(Tracer, die),
 
     Tracer2 = spawn_link(fun receiver/0),
     timer:sleep(1),
-    N = erlang:trace(existing, true, [send, {tracer, Tracer2}]),
-    {flags, [send]} = erlang:trace_info(Proc, flags),
-    {tracer, Tracer2} = erlang:trace_info(Proc, tracer),
+    N = erlang_trace(existing, true, [send, {tracer, Tracer2}]),
+    {flags, [send]} = erlang_trace_info(Proc, flags),
+    {tracer, Tracer2} = erlang_trace_info(Proc, tracer),
     unlink(Tracer2),
     exit(Tracer2, die),
 
     Tracer3 = spawn_link(fun receiver/0),
     timer:sleep(1),
-    1 = erlang:trace(Proc, true, [send, {tracer, Tracer3}]),
-    {flags, [send]} = erlang:trace_info(Proc, flags),
-    {tracer, Tracer3} = erlang:trace_info(Proc, tracer),
+    1 = erlang_trace(Proc, true, [send, {tracer, Tracer3}]),
+    {flags, [send]} = erlang_trace_info(Proc, flags),
+    {tracer, Tracer3} = erlang_trace_info(Proc, tracer),
     unlink(Tracer3),
     exit(Tracer3, die),
 
@@ -1719,20 +1834,20 @@ tracer_die(Config) when is_list(Config) ->
 %% Test that an invalid flag cause badarg
 bad_flag(Config) when is_list(Config) ->
     %% A bad flag could deadlock the SMP emulator in erts-5.5
-    {'EXIT', {badarg, _}} = (catch erlang:trace(new,
+    {'EXIT', {badarg, _}} = (catch erlang_trace(new,
                                                 true,
                                                 [not_a_valid_flag])),
 
     %% Leaks of {tracer,_} in OTP 23.2
     Pid = spawn(fun() -> receive die -> ok end end),
-    1 = erlang:trace(Pid, true, [{tracer, self()},
+    1 = erlang_trace(Pid, true, [{tracer, self()},
                                  {tracer, self()}]),
     Pid ! die,
     {'EXIT', {badarg, _}} =
-        (catch erlang:trace(new, true, [{tracer, self()}
+        (catch erlang_trace(new, true, [{tracer, self()}
                                         | improper])),
     {'EXIT', {badarg, _}} =
-        (catch erlang:trace(new, true, [{tracer, self()},
+        (catch erlang_trace(new, true, [{tracer, self()},
                                         not_a_valid_flag])),
     ok.
 
@@ -1746,7 +1861,7 @@ trace_delivered(Config) when is_list(Config) ->
                         receive Go -> gone end,
                         tok_trace_loop(Parent, 0, TokLoops)
                 end),
-    1 = erlang:trace(Tok, true, [procs]),
+    1 = erlang_trace(Tok, true, [procs]),
     Mon = erlang:monitor(process, Tok),
     NoOfTraceMessages = 4*TokLoops + 1,
     io:format("Expect a total of ~p trace messages~n",
@@ -1766,7 +1881,7 @@ trap_exit_self_receive(Config) when is_list(Config) ->
     Parent = self(),
     Proc = spawn_link(fun() -> process(Parent) end),
 
-    1 = erlang:trace(Proc, true, ['receive']),
+    1 = erlang_trace(Proc, true, ['receive']),
     Proc ! {trap_exit_please, true},
     {trace, Proc, 'receive', {trap_exit_please, true}} = receive_first_trace(),
 
@@ -1786,7 +1901,7 @@ trap_exit_self_receive(Config) when is_list(Config) ->
     ok.
 
 trace_info_badarg(Config) when is_list(Config) ->
-    catch erlang:trace_info({a,b,c},d),
+    catch erlang_trace_info({a,b,c},d),
     ok.
 
 %% An incoming suspend monitor down wasn't handled
@@ -1812,7 +1927,7 @@ ms_excessive_nesting(Config) when is_list(Config) ->
     %% (hmm...) nesting
     MS = [{['$1'], [MkMSCond(MkMSCond, 100)], []}],
     io:format("~p~n", [erlang:match_spec_test([1], MS, trace)]),
-    _ = erlang:trace_pattern({?MODULE, '_', '_'}, MS, []),
+    _ = erlang_trace_pattern({?MODULE, '_', '_'}, MS, []),
     %% Now test a match spec using excessive nesting. This
     %% used to seg-fault the emulator due to recursion
     %% beyond the end of the C-stack.
@@ -1824,7 +1939,7 @@ ms_excessive_nesting(Config) when is_list(Config) ->
     ENMS = [{['$1'], [MkMSCond(MkMSCond, 1000000)], []}],
     io:format("~p~n", [erlang:match_spec_test([1], ENMS, trace)]),
     try
-        _ = erlang:trace_pattern({?MODULE, '_', '_'}, ENMS, []),
+        _ = erlang_trace_pattern({?MODULE, '_', '_'}, ENMS, []),
         {comment, "compiled"}
     catch
         error:system_limit ->
@@ -1864,6 +1979,8 @@ tok_trace_loop(Parent, N, M) ->
 receive_first() ->
     receive
         Any -> Any
+    after 1000 ->
+            timeout
     end.
 
 %% Waits for and returns the first message in the message queue.
@@ -1871,6 +1988,8 @@ receive_first() ->
 receive_first_trace() ->
     receive
 	Any when element(1,Any) =:= trace; element(1,Any) =:= trace_ts -> Any
+    after 1000 ->
+            timeout
     end.
 
 %% Ensures that there is no message in the message queue.
