@@ -16,14 +16,12 @@
 %%
 %% %CopyrightEnd%
 %%
-%% This module tests that beam_ssa_alias_opt:to_private_append/3
+%% This module tests that the beam_ssa_destructive_update pass
 %% rewrites plain appends in bs_create_bin to private_append when
 %% appropriate.
 %%
 -module(private_append).
 
--feature(maybe_expr, enable).
-
 -export([transformable0/1,
 	 transformable1/1,
 	 transformable1b/1,
@@ -83,8 +81,7 @@
          bs_create_bin_on_literal/0,
 
          crash_in_value_tracking/3,
-         crash_in_value_tracking_inner/3,
-         gh8630/1]).
+         crash_in_value_tracking_inner/3]).
 
 %% Trivial smoke test
 transformable0(L) ->
@@ -200,15 +197,18 @@ transformable4([H|T], Acc) ->
 transformable4([], Acc) ->
     Acc.
 
-%% Check that the alias analysis handles local functions.
+%% Check that when a variable is live across a call it is considered
+%% aliased. If the alias analysis is extended to track if an argument
+%% is captured by the callee, that information could be fed back to
+%% the caller. Until that is done, this test is expected to fail.
 transformable5(L) ->
-%ssa% (_) when post_ssa_opt ->
+%ssa% xfail (_) when post_ssa_opt ->
 %ssa% A = bs_init_writable(_),
 %ssa% _ = call(fun transformable5/2, _, A).
     transformable5(L, <<>>).
 
 transformable5([H|T], Acc) ->
-%ssa% (_, Arg1) when post_ssa_opt ->
+%ssa% xfail (_, Arg1) when post_ssa_opt ->
 %ssa% A = bs_create_bin(private_append, _, Arg1, ...),
 %ssa% _ = call(fun transformable5/2, _, A).
     does_not_escape(Acc),
@@ -443,18 +443,17 @@ transformable16([A,B|T], {{Acc0}, Acc1}) ->
 transformable16([], {{Acc0}, Acc1}) ->
     {Acc0,Acc1}.
 
-%% We should use type information to figure out that {<<>>, X} is not
-%% aliased, but as of now we don't have the information at this pass,
-%% nor do we track alias status at the sub-term level.
+%% Check that type information is used to figure out that {<<>>, X} is
+%% not aliased.
 transformable18(L, X) when is_integer(X), X < 256 ->
-%ssa% xfail (_, _) when post_ssa_opt ->
+%ssa% (_, _) when post_ssa_opt ->
 %ssa% A = bs_init_writable(_),
-%ssa% B = put_tuple(_, A),
+%ssa% B = put_tuple(A, _),
 %ssa% _ = call(fun transformable18b/2, _, B).
     transformable18b(L, {<<>>, X}).
 
 transformable18b([H|T], {Acc,X}) ->
-%ssa% xfail (_, Arg1) when post_ssa_opt ->
+%ssa% (_, Arg1) when post_ssa_opt ->
 %ssa% A = get_tuple_element(Arg1, 0),
 %ssa% B = bs_create_bin(private_append, _, A, ...),
 %ssa% C = put_tuple(B, _),
@@ -595,16 +594,15 @@ make_empty_binary_tuple_nested() ->
 %ssa% ret(D).
     {<<>>, {<<>>}, 47}.
 
-%% We can't handle this as we do not track alias status at the sub-term level.
 transformable24(L) ->
-%ssa% xfail (_) when post_ssa_opt ->
+%ssa% (_) when post_ssa_opt ->
 %ssa% A = bs_init_writable(_),
 %ssa% B = put_tuple(A, _),
 %ssa% _ = call(fun transformable24/2, _, B).
     transformable24(L, {<<>>, ex:foo()}).
 
 transformable24([H|T], {Acc,X}) ->
-%ssa% xfail (_, Arg1) when post_ssa_opt ->
+%ssa% (_, Arg1) when post_ssa_opt ->
 %ssa% A = get_tuple_element(Arg1, 0),
 %ssa% B = bs_create_bin(private_append, _, A, ...),
 %ssa% C = put_tuple(B, _),
@@ -772,6 +770,17 @@ transformable33() ->
 transformable33_inner(V) ->
     << <<C>> || <<C:4>> <= V >>.
 
+%% Check that calling an external function with append result doesn't
+%% prevent private_append optimization.
+transformable34() ->
+    transformable34a(<<>>).
+
+transformable34a(Acc) ->
+%ssa% (Arg1) when post_ssa_opt ->
+%ssa% _ = bs_create_bin(private_append, _, Arg1, ...).
+    Value = <<Acc/binary, 1>>,
+    ex:escape(Value).
+
 % Should not be transformed as we can't know the alias status of Acc
 not_transformable1([H|T], Acc) ->
 %ssa% (_, Arg1) when post_ssa_opt ->
@@ -1006,13 +1015,13 @@ bs_create_bin_on_literal() ->
       >>/binary
     >>.
 
-%% Check that the beam_ssa_private_append pass doesn't crash, if it,
-%% during initial value tracking, ends up in operations which do not
-%% create bit strings. This can happen as the initial value tracking
-%% in beam_ssa_private_append doesn't consider types. As the decision
-%% to apply the private append transform is using type information,
-%% tracking values into not type-compatible execution paths is
-%% harmless.
+%% Check that the beam_ssa_destructive_update pass doesn't crash, if
+%% it, during initial value tracking, ends up in operations which do
+%% not create bit strings. This can happen as the initial value
+%% tracking in beam_ssa_destructive_update doesn't consider types. As
+%% the decision to apply the private append transform is using type
+%% information, tracking values into not type-compatible execution
+%% paths is harmless.
 crash_in_value_tracking_inner(_, 1.0, _) ->
 %ssa% (_, _, _) when post_ssa_opt ->
 %ssa% _ = bs_init_writable(_).
@@ -1026,13 +1035,3 @@ crash_in_value_tracking(_, _V0, _) ->
     ((<<((crash_in_value_tracking_inner(
             {#{#{ ok => ok || _ := _ <- ok} => ok},
              _V0, false, _V0, "Bo"}, _V0, ok)))/bytes>>) =/= ok).
-
-gh8630(<<"\\",R/binary>>, Xs) ->
-%ssa% (_, _) when post_ssa_opt ->
-%ssa% _ = bs_init_writable(_).
-    gh8630(R, [<<>> | Xs]);
-gh8630(<<A:8,R/binary>>, [X|Xs]) ->
-    gh8630(R, [<<X/binary,A:8>> | Xs]).
-
-gh8630(I) ->
-    gh8630(I, []).
