@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2010-2024. All Rights Reserved.
+ * Copyright Ericsson AB 2010-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,7 +29,7 @@
 #ifdef HAS_3_0_API
 #else
 static unsigned int algo_hash_cnt, algo_hash_fips_cnt;
-static ERL_NIF_TERM algo_hash[16];   /* increase when extending the list */
+static ERL_NIF_TERM algo_hash[17];   /* increase when extending the list */
 void init_hash_types(ErlNifEnv* env);
 #endif
 
@@ -39,7 +39,8 @@ void init_pubkey_types(ErlNifEnv* env);
 
 static ERL_NIF_TERM algo_curve[2][89]; /* increase when extending the list */
 static ErlNifMutex* mtx_init_curve_types;
-static int get_curve_cnt(ErlNifEnv* env, int fips);
+void init_curve_types(ErlNifEnv* env);
+int get_curve_cnt(ErlNifEnv* env, int fips);
 
 static unsigned int algo_rsa_opts_cnt, algo_rsa_opts_fips_cnt;
 static ERL_NIF_TERM algo_rsa_opts[11]; /* increase when extending the list */
@@ -55,6 +56,7 @@ void init_algorithms_types(ErlNifEnv* env)
     init_hash_types(env);
 #endif
     init_pubkey_types(env);
+    init_curve_types(env);
     init_rsa_opts_types(env);
     /* ciphers and macs are initiated statically */
 }
@@ -128,6 +130,9 @@ void init_hash_types(ErlNifEnv* env) {
 #ifdef HAVE_SHAKE256
     algo_hash[algo_hash_cnt++] = enif_make_atom(env, "shake256");
 #endif
+#ifdef HAVE_SM3
+    algo_hash[algo_hash_cnt++] = enif_make_atom(env, "sm3");
+#endif
 #ifdef HAVE_BLAKE2
     algo_hash[algo_hash_cnt++] = enif_make_atom(env, "blake2b");
     algo_hash[algo_hash_cnt++] = enif_make_atom(env, "blake2s");
@@ -228,9 +233,9 @@ ERL_NIF_TERM curve_algorithms(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[
     return enif_make_list_from_array(env, algo_curve[fips_mode], algo_curve_cnt);
 }
 
-static int init_curves(ErlNifEnv* env, int fips);
+int init_curves(ErlNifEnv* env, int fips);
 #if defined(HAVE_EC)
-static int valid_curve(int nid);
+int valid_curve(int nid);
 #endif
 
 int get_curve_cnt(ErlNifEnv* env, int fips) {
@@ -247,14 +252,16 @@ int get_curve_cnt(ErlNifEnv* env, int fips) {
 
     enif_mutex_lock(mtx_init_curve_types);
     if (1 == fips) {
-        if (algo_curve_fips_cnt < 0) {
-            algo_curve_fips_cnt = init_curves(env, 1);
+        if (algo_curve_fips_cnt >= 0) {
+            return algo_curve_fips_cnt;
         }
+        algo_curve_fips_cnt = init_curves(env, 1);
         cnt = algo_curve_fips_cnt;
     } else {
-        if (algo_curve_cnt < 0) {
-            algo_curve_cnt = init_curves(env, 0);
+        if (algo_curve_cnt >= 0) {
+            return algo_curve_cnt;
         }
+        algo_curve_cnt = init_curves(env, 0);
         cnt = algo_curve_cnt;
     }
     enif_mutex_unlock(mtx_init_curve_types);
@@ -262,6 +269,38 @@ int get_curve_cnt(ErlNifEnv* env, int fips) {
     return cnt;
 }
 
+void init_curve_types(ErlNifEnv* env) {
+    /* Initialize the curve counters and curve's lists
+       by calling get_curve_cnt
+    */
+#ifdef FIPS_SUPPORT
+    if (FIPS_MODE()) {
+        // FIPS enabled
+        get_curve_cnt(env, 1);
+        FIPS_mode_set(0); // disable
+        get_curve_cnt(env, 0);
+        FIPS_mode_set(1); // re-enable
+    } else {
+        // FIPS disabled but available
+        get_curve_cnt(env, 0);
+        FIPS_mode_set(1); // enable
+        get_curve_cnt(env, 1);
+        FIPS_mode_set(0); // re-disable
+    }
+#else
+    // FIPS mode is not available
+    get_curve_cnt(env, 0);
+#endif
+
+# ifdef DEBUG
+    {
+        int curve_cnt = get_curve_cnt(env, 0);
+        ASSERT(curve_cnt <= sizeof(algo_curve[0])/sizeof(ERL_NIF_TERM));
+    }
+# endif 
+}
+
+
 int init_curves(ErlNifEnv* env, int fips) {
 #if defined(HAVE_EC)
     int cnt = 0;
@@ -611,8 +650,6 @@ int init_curves(ErlNifEnv* env, int fips) {
 #endif
     }
 
-    ASSERT(cnt <= sizeof(algo_curve[0])/sizeof(ERL_NIF_TERM));
-
     return cnt;
 #else /* if not HAVE_EC */
     return 0;
