@@ -24,6 +24,7 @@
 %%----------------------------------------------------------------------
 
 -module(tls_v1).
+-moduledoc false.
 
 -include("ssl_cipher.hrl").
 -include("ssl_internal.hrl").
@@ -39,6 +40,7 @@
          suites/1,
          exclusive_suites/1,
          exclusive_anonymous_suites/1,
+         cbc_suites/1,
          psk_suites/1,
          psk_exclusive/1,
          psk_suites_anon/1,
@@ -502,15 +504,51 @@ mac_hash(Method, Mac_write_secret, Seq_num, Type, Version,Length, Fragment) ->
 -spec suites(ssl_record:ssl_version()) -> [ssl_cipher_format:cipher_suite()].
 
 suites(Version) when ?TLS_1_X(Version) ->
-    lists:flatmap(fun exclusive_suites/1, suites_to_test(Version)).
+    lists:flatmap(fun default_suites/1, suites_in_version(Version)).
 
-suites_to_test(?TLS_1_0) -> [?TLS_1_0];
-suites_to_test(?TLS_1_1) -> [?TLS_1_0];
-suites_to_test(?TLS_1_2) -> [?TLS_1_2, ?TLS_1_0];
-suites_to_test(?TLS_1_3) -> [?TLS_1_3, ?TLS_1_2, ?TLS_1_0].
+suites_in_version(?TLS_1_0) -> [?TLS_1_0];
+suites_in_version(?TLS_1_1) -> [?TLS_1_0];
+suites_in_version(?TLS_1_2) -> [?TLS_1_2];
+suites_in_version(?TLS_1_3) -> [?TLS_1_3, ?TLS_1_2].
 
 -spec exclusive_suites(ssl_record:ssl_version()) -> [ssl_cipher_format:cipher_suite()].
 
+default_suites(?TLS_1_3 = Version) ->
+    exclusive_suites(Version);
+default_suites(?TLS_1_2) ->
+    [?TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
+     ?TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
+
+     ?TLS_ECDHE_ECDSA_WITH_AES_256_CCM,
+     ?TLS_ECDHE_ECDSA_WITH_AES_256_CCM_8,
+
+     ?TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,
+     ?TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
+
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
+     ?TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
+
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CCM,
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CCM_8,
+
+     ?TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384,
+     ?TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384,
+
+     ?TLS_ECDH_ECDSA_WITH_AES_128_GCM_SHA256,
+     ?TLS_ECDH_RSA_WITH_AES_128_GCM_SHA256,
+
+     ?TLS_DHE_RSA_WITH_AES_256_GCM_SHA384,
+     ?TLS_DHE_DSS_WITH_AES_256_GCM_SHA384,
+
+     ?TLS_DHE_RSA_WITH_AES_128_GCM_SHA256,
+     ?TLS_DHE_DSS_WITH_AES_128_GCM_SHA256,
+
+     ?TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256
+    ];
+default_suites(Version) when Version == ?TLS_1_1;
+                             Version == ?TLS_1_0 ->
+    exclusive_suites(?TLS_1_0).
+
 exclusive_suites(?TLS_1_3) ->
     [?TLS_AES_256_GCM_SHA384,
      ?TLS_AES_128_GCM_SHA256,
@@ -527,9 +565,6 @@ exclusive_suites(?TLS_1_2) ->
      ?TLS_ECDHE_ECDSA_WITH_AES_256_CCM,
      ?TLS_ECDHE_ECDSA_WITH_AES_256_CCM_8,
 
-     ?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384,
-     ?TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384,
-
      ?TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,
      ?TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
 
@@ -542,32 +577,16 @@ exclusive_suites(?TLS_1_2) ->
      ?TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384,
      ?TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384,
 
-     ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384,
-     ?TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384,
-
      ?TLS_ECDH_ECDSA_WITH_AES_128_GCM_SHA256,
      ?TLS_ECDH_RSA_WITH_AES_128_GCM_SHA256,
 
-     ?TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
-     ?TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
-
-     ?TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256,
-     ?TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256,
-
      ?TLS_DHE_RSA_WITH_AES_256_GCM_SHA384,
      ?TLS_DHE_DSS_WITH_AES_256_GCM_SHA384,
 
-     ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA256,
-     ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA256,
-
      ?TLS_DHE_RSA_WITH_AES_128_GCM_SHA256,
      ?TLS_DHE_DSS_WITH_AES_128_GCM_SHA256,
 
-     ?TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
-
-     ?TLS_DHE_RSA_WITH_AES_128_CBC_SHA256,
-     ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA256
-
+     ?TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256
      %% not supported
      %% ?TLS_DH_RSA_WITH_AES_256_GCM_SHA384,
      %% ?TLS_DH_DSS_WITH_AES_256_GCM_SHA384,
@@ -577,8 +596,7 @@ exclusive_suites(?TLS_1_2) ->
 exclusive_suites(?TLS_1_1) ->
     [];
 exclusive_suites(?TLS_1_0) ->
-    [
-     ?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
+    [?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA,
      ?TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA,
 
      ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
@@ -593,8 +611,7 @@ exclusive_suites(?TLS_1_0) ->
      ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA,
      ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA,
      ?TLS_DHE_RSA_WITH_AES_128_CBC_SHA,
-     ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA
-    ].
+     ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA].
 
 %%--------------------------------------------------------------------
 -spec exclusive_anonymous_suites(ssl_record:ssl_version()) ->
@@ -632,6 +649,31 @@ exclusive_anonymous_suites(?TLS_1_0=Version) ->
          ?TLS_DH_anon_WITH_DES_CBC_SHA
         ] ++ srp_suites_anon(Version).
 
+
+cbc_suites(Version) when ?TLS_1_X(Version) ->
+    cbc_exclusive(Version).
+
+cbc_exclusive(?TLS_1_2) ->
+    [?TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384,
+     ?TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384,
+     ?TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384,
+     ?TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384,
+     ?TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
+     ?TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
+     ?TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256,
+     ?TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256,
+     ?TLS_DHE_RSA_WITH_AES_256_CBC_SHA256,
+     ?TLS_DHE_DSS_WITH_AES_256_CBC_SHA256,
+     ?TLS_DHE_RSA_WITH_AES_128_CBC_SHA256,
+     ?TLS_DHE_DSS_WITH_AES_128_CBC_SHA256
+    ];
+cbc_exclusive(?TLS_1_1) ->
+    %% Only have CBC SUITES
+    %% disabled even though they are legacy
+    [];
+cbc_exclusive(?TLS_1_0) ->
+    [?TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA].
+
 %%--------------------------------------------------------------------
 -spec psk_suites(ssl_record:ssl_version()) -> [ssl_cipher_format:cipher_suite()].
 %%
@@ -813,11 +855,11 @@ des_exclusive(_) ->
 %% Are not considered secure any more.
 %%--------------------------------------------------------------------
 rsa_suites(Version) when ?TLS_1_X(Version) ->
-    lists:flatmap(fun rsa_exclusive/1, rsa_suites_to_test(Version)).
+    lists:flatmap(fun rsa_exclusive/1, rsa_suites_in_version(Version)).
 
-rsa_suites_to_test(?TLS_1_2) -> [?TLS_1_2, ?TLS_1_0];
-rsa_suites_to_test(?TLS_1_1) -> [?TLS_1_0];
-rsa_suites_to_test(?TLS_1_0) -> [?TLS_1_0].
+rsa_suites_in_version(?TLS_1_2) -> [?TLS_1_2, ?TLS_1_0];
+rsa_suites_in_version(?TLS_1_1) -> [?TLS_1_0];
+rsa_suites_in_version(?TLS_1_0) -> [?TLS_1_0].
 
 -spec rsa_exclusive(Version::ssl_record:ssl_version()) -> [ssl_cipher_format:cipher_suite()].
 rsa_exclusive(?TLS_1_2) ->
@@ -976,6 +1018,9 @@ default_signature_schemes(Version) ->
                ecdsa_secp521r1_sha512,
                ecdsa_secp384r1_sha384,
                ecdsa_secp256r1_sha256,
+               ecdsa_brainpoolP512r1tls13_sha512,
+               ecdsa_brainpoolP384r1tls13_sha384,
+               ecdsa_brainpoolP256r1tls13_sha256,
                rsa_pss_pss_sha512,
                rsa_pss_pss_sha384,
                rsa_pss_pss_sha256,
@@ -1164,9 +1209,12 @@ groups() ->
 groups(all) ->
     [x25519,
      x448,
-     secp256r1,
-     secp384r1,
      secp521r1,
+     secp384r1,
+     secp256r1,
+     brainpoolP256r1tls13,
+     brainpoolP384r1tls13,
+     brainpoolP512r1tls13,
      ffdhe2048,
      ffdhe3072,
      ffdhe4096,
@@ -1175,17 +1223,22 @@ groups(all) ->
 groups(default) ->
     [x25519,
      x448,
+     secp521r1,
+     secp384r1,
      secp256r1,
-     secp384r1];
+     brainpoolP512r1tls13,
+     brainpoolP384r1tls13,
+     brainpoolP256r1tls13
+    ];
 groups(TLSGroups) when is_list(TLSGroups) ->
-    CryptoGroups = supported_groups(),
-    lists:filter(fun(Group) -> proplists:get_bool(Group, CryptoGroups) end, TLSGroups).
+    CryptoGroups = crypto_supported_groups(),
+    lists:filter(fun(Group) -> proplists:get_bool(maybe_group_to_curve(Group), CryptoGroups) end, TLSGroups).
 
 default_groups() ->
     TLSGroups = groups(default),
     groups(TLSGroups).
 
-supported_groups() ->
+crypto_supported_groups() ->
     crypto:supports(curves) ++
         [ffdhe2048,ffdhe3072,ffdhe4096,ffdhe6144,ffdhe8192].
 
@@ -1194,6 +1247,9 @@ group_to_enum(secp384r1) -> ?SECP384R1;
 group_to_enum(secp521r1) -> ?SECP521R1;
 group_to_enum(x25519)    -> ?X25519;
 group_to_enum(x448)      -> ?X448;
+group_to_enum(brainpoolP256r1tls13) -> ?BRAINPOOLP256R1TLS13;
+group_to_enum(brainpoolP384r1tls13) -> ?BRAINPOOLP384R1TLS13;
+group_to_enum(brainpoolP512r1tls13) -> ?BRAINPOOLP512R1TLS13;
 group_to_enum(ffdhe2048) -> ?FFDHE2048;
 group_to_enum(ffdhe3072) -> ?FFDHE3072;
 group_to_enum(ffdhe4096) -> ?FFDHE4096;
@@ -1205,6 +1261,9 @@ enum_to_group(?SECP384R1) -> secp384r1;
 enum_to_group(?SECP521R1) -> secp521r1;
 enum_to_group(?X25519) -> x25519;
 enum_to_group(?X448) -> x448;
+enum_to_group(?BRAINPOOLP256R1TLS13) -> brainpoolP256r1tls13;
+enum_to_group(?BRAINPOOLP384R1TLS13) -> brainpoolP384r1tls13;
+enum_to_group(?BRAINPOOLP512R1TLS13) -> brainpoolP512r1tls13;
 enum_to_group(?FFDHE2048) -> ffdhe2048;
 enum_to_group(?FFDHE3072) -> ffdhe3072;
 enum_to_group(?FFDHE4096) -> ffdhe4096;
@@ -1212,7 +1271,8 @@ enum_to_group(?FFDHE6144) -> ffdhe6144;
 enum_to_group(?FFDHE8192) -> ffdhe8192;
 enum_to_group(_) -> undefined.
 
-%% 1-22 deprecated in RFC 8422
+
+%% 1-22 Deprecated in RFC 8422
 oid_to_enum(?sect163k1) -> 1;
 oid_to_enum(?sect163r1) -> 2;
 oid_to_enum(?sect163r2) -> 3;
@@ -1235,10 +1295,12 @@ oid_to_enum(?secp192r1) -> 19;
 oid_to_enum(?secp224k1) -> 20;
 oid_to_enum(?secp224r1) -> 21;
 oid_to_enum(?secp256k1) -> 22;
+
 %% RFC 8422
 oid_to_enum(?secp256r1) -> 23;
 oid_to_enum(?secp384r1) -> 24;
 oid_to_enum(?secp521r1) -> 25;
+
 %% RFC 7027
 oid_to_enum(?brainpoolP256r1) -> 26;
 oid_to_enum(?brainpoolP384r1) -> 27;
@@ -1280,6 +1342,16 @@ enum_to_oid(30) -> ?'id-X448';
 enum_to_oid(_) ->
     undefined.
 
+maybe_group_to_curve(brainpoolP512r1tls13) ->
+    brainpoolP512r1;
+maybe_group_to_curve(brainpoolP384r1tls13) ->
+    brainpoolP384r1;
+maybe_group_to_curve(brainpoolP256r1tls13) ->
+    brainpoolP256r1;
+maybe_group_to_curve(Group) ->
+    Group.
+
+
 %%%################################################################
 %%%#
 %%%# Tracing
