@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2009-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2009-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -29,9 +29,11 @@
          file_read_stdin_binary_mode/1, file_read_stdin_list_mode/1,
          file_read_stdin_unicode_translation_error_binary_mode/1,
          file_read_stdin_unicode_translation_error_list_mode/1,
+         file_read_line_stdin_cr_without_nl/1,
          file_read_line_stdin_unicode_translation_error_binary_mode/1,
          file_read_line_stdin_unicode_translation_error_list_mode/1,
          io_get_chars_stdin_binary_mode/1, io_get_chars_stdin_list_mode/1,
+         io_get_until_stdin_binary_mode/1, io_get_until_stdin_list_mode/1,
          io_get_chars_file_read_stdin_binary_mode/1,
          file_read_stdin_latin1_binary_mode/1,
          file_read_stdin_latin1_list_mode/1,
@@ -48,6 +50,10 @@
 
 -export([write_raw_to_stdout/0, read_raw_from_stdin/1]).
 
+-export([get_until_eof/2]).
+
+-include_lib("stdlib/include/assert.hrl").
+
 %%-define(debug, true).
 
 -ifdef(debug).
@@ -73,6 +79,8 @@ all() ->
      file_read_line_stdin_unicode_translation_error_list_mode,
      io_get_chars_stdin_binary_mode,
      io_get_chars_stdin_list_mode,
+     io_get_until_stdin_binary_mode,
+     io_get_until_stdin_list_mode,
      io_get_chars_file_read_stdin_binary_mode,
      file_read_stdin_latin1_binary_mode,
      file_read_stdin_latin1_list_mode,
@@ -270,7 +278,15 @@ setopts_getopts(Config) when is_list(Config) ->
                {expect, "[\n ]ok"},
                {putline, "io:get_line('')."},
                {putline, "hej"},
-               {expect, "\\Q<<\"hej\\n\">>\\E"}
+               {expect, "\\Q<<\"hej\\n\">>\\E"},
+               {putline, "proplists:get_value(terminal,io:getopts())."},
+               {expect, "true"},
+               {putline, "proplists:get_value(stdin,io:getopts())."},
+               {expect, "true"},
+               {putline, "proplists:get_value(stdout,io:getopts())."},
+               {expect, "true"},
+               {putline, "proplists:get_value(stderr,io:getopts())."},
+               {expect, "true"}
               ],[]);
         _ ->
             ok
@@ -289,8 +305,31 @@ setopts_getopts(Config) when is_list(Config) ->
        {expect, "[\n ]ok"},
        {putline, "io:get_line('')."},
        {putline, "hej"},
-       {expect, "\\Q<<\"hej\\n\">>\\E"}
+       {expect, "\\Q<<\"hej\\n\">>\\E"},
+       {putline, "proplists:get_value(terminal,io:getopts())."},
+       {expect, "true"},
+       {putline, "proplists:get_value(stdin,io:getopts())."},
+       {expect, "true"},
+       {putline, "proplists:get_value(stdout,io:getopts())."},
+       {expect, "true"},
+       {putline, "proplists:get_value(stderr,io:getopts())."},
+       {expect, "true"}
       ],[],"",["-oldshell"]),
+
+    %% Test that terminal options when used in non-terminal
+    %% are returned as they should
+    Erl = ct:get_progname(),
+    Str = os:cmd(Erl ++ " -noshell -eval \"io:format(~s'~p.',[io:getopts()])\" -s init stop"),
+    maybe
+        {ok, T, _} ?= erl_scan:string(Str),
+        {ok, Opts} ?= erl_parse:parse_term(T),
+        ?assertEqual(false, proplists:get_value(terminal,Opts)),
+        ?assertEqual(false, proplists:get_value(stdin,Opts)),
+        ?assertEqual(false, proplists:get_value(stdout,Opts)),
+        ?assertEqual(false, proplists:get_value(stderr,Opts))
+    else
+        _ -> ct:fail({failed_to_parse, Str})
+    end,
     ok.
 
 %% Test that reading from stdin using file:read works when io is in binary mode
@@ -343,6 +382,20 @@ file_read_stdin_unicode_translation_error_list_mode(_Config) ->
 
     ok.
 
+%% Test that reading from stdin using file:read_line works when \r is sent without \n
+file_read_line_stdin_cr_without_nl(_Config) ->
+    {ok, P, ErlPort} = start_stdin_node(fun() -> file:read_line(standard_io) end, []),
+
+    erlang:port_command(ErlPort, "abc\r"),
+    {error,timeout} = gen_tcp:recv(P, 0, 2000),
+    erlang:port_command(ErlPort, "def\r\n"),
+    {ok, ~S'got: <<"abc\rdef\r\n">>\n'} = gen_tcp:recv(P, 0),
+    ErlPort ! {self(), close},
+    {ok, "got: eof"} = gen_tcp:recv(P, 0),
+
+    ok.
+
+
 %% Test that reading from stdin using file:read_line returns
 %% correct error when in binary mode
 file_read_line_stdin_unicode_translation_error_binary_mode(_Config) ->
@@ -367,7 +420,7 @@ file_read_line_stdin_unicode_translation_error_list_mode(_Config) ->
 
     ok.
 
-%% Test that reading from stdin using file:read works when io is in binary mode
+%% Test that reading from stdin using io:get_chars works when io is in binary mode
 io_get_chars_stdin_binary_mode(_Config) ->
     {ok, P, ErlPort} = start_stdin_node(
                          fun() ->
@@ -385,7 +438,75 @@ io_get_chars_stdin_binary_mode(_Config) ->
 
     ok.
 
-%% Test that reading from stdin using file:read works when io is in binary mode
+%% Test that reading from stdin using custom io_request works when io is in binary mode
+io_get_until_stdin_binary_mode(_Config) ->
+
+    GetUntilEof =
+        fun() ->
+                IoServer = group_leader(),
+                IoServer !
+                    {io_request,
+                     self(),
+                     IoServer,
+                     {get_until, unicode, '', ?MODULE, get_until_eof, []}},
+                receive
+                    {io_reply, IoServer, Data} ->
+                        {ok, Data}
+                end
+        end,
+
+    {ok, P, ErlPort} = start_stdin_node(GetUntilEof, [binary]),
+
+    erlang:port_command(ErlPort, "x\n"),
+    {error, timeout} = gen_tcp:recv(P, 0, 250),
+    ErlPort ! {self(), close},
+    {ok, "got: <<\"x\\n\">>\n"} = gen_tcp:recv(P, 0),
+
+    {ok, P2, ErlPort2} = start_stdin_node(GetUntilEof, [binary]),
+
+    {error, timeout} = gen_tcp:recv(P2, 0, 250),
+    ErlPort2 ! {self(), close},
+    {ok, "got: eof\n"} = gen_tcp:recv(P2, 0),
+
+    ok.
+
+%% Test that reading from stdin using custom io_request works when io is in list mode
+io_get_until_stdin_list_mode(_Config) ->
+
+    GetUntilEof =
+        fun() ->
+                IoServer = group_leader(),
+                IoServer !
+                    {io_request,
+                     self(),
+                     IoServer,
+                     {get_until, unicode, '', ?MODULE, get_until_eof, []}},
+                receive
+                    {io_reply, IoServer, Data} ->
+                        {ok, Data}
+                end
+        end,
+
+    {ok, P, ErlPort} = start_stdin_node(GetUntilEof, [list]),
+
+    erlang:port_command(ErlPort, "x\n"),
+    {error, timeout} = gen_tcp:recv(P, 0, 250),
+    ErlPort ! {self(), close},
+    {ok, "got: \"x\\n\"\n"} = gen_tcp:recv(P, 0),
+
+    {ok, P2, ErlPort2} = start_stdin_node(GetUntilEof, [list]),
+
+    {error, timeout} = gen_tcp:recv(P2, 0, 250),
+    ErlPort2 ! {self(), close},
+    {ok, "got: eof\n"} = gen_tcp:recv(P2, 0),
+
+    ok.
+
+get_until_eof([],eof) -> {done,eof,[]};
+get_until_eof(ThisFar,eof) -> {done,ThisFar,eof};
+get_until_eof(ThisFar,CharList) -> {more,ThisFar++CharList}.
+
+%% Test that reading from stdin using io:get_chars works when io is in list mode
 io_get_chars_stdin_list_mode(_Config) ->
     {ok, P, ErlPort} = start_stdin_node(
                          fun() -> case io:get_chars(standard_io, "", 1) of
