@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2006-2025. All Rights Reserved.
+%% Copyright Ericsson AB 2006-2023. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -22,12 +22,13 @@
 -export([all/0, suite/0,groups/0,init_per_suite/1, end_per_suite/1, 
 	 init_per_group/2,end_per_group/2, borderline/1, atomic/1,
          bad_zip/1, unzip_from_binary/1, unzip_to_binary/1,
-         zip_to_binary/1, sanitize_filenames/1,
+         zip_to_binary/1,
          unzip_options/1, zip_options/1, list_dir_options/1, aliases/1,
          openzip_api/1, zip_api/1, open_leak/1, unzip_jar/1,
 	 unzip_traversal_exploit/1,
          compress_control/1,
-	 foldl/1,fd_leak/1,unicode/1,test_zip_dir/1]).
+	 foldl/1,fd_leak/1,unicode/1,test_zip_dir/1,
+         explicit_file_info/1]).
 
 -include_lib("common_test/include/ct.hrl").
 -include_lib("kernel/include/file.hrl").
@@ -41,7 +42,7 @@ all() ->
      zip_options, list_dir_options, aliases, openzip_api,
      zip_api, open_leak, unzip_jar, compress_control, foldl,
      unzip_traversal_exploit,fd_leak,unicode,test_zip_dir,
-     sanitize_filenames].
+     explicit_file_info].
 
 groups() -> 
     [].
@@ -91,27 +92,22 @@ borderline_test(Size, TempDir) ->
     {ok, Archive} = zip:zip(Archive, [Name]),
     ok = file:delete(Name),
 
-    RelName = filename:join(tl(filename:split(Name))),
-
     %% Verify listing and extracting.
     {ok, [#zip_comment{comment = []},
-          #zip_file{name = RelName,
+          #zip_file{name = Name,
                     info = Info,
                     offset = 0,
                     comp_size = _}]} = zip:list_dir(Archive),
     Size = Info#file_info.size,
-    TempRelName = filename:join(TempDir, RelName),
-    {ok, [TempRelName]} = zip:extract(Archive, [verbose, {cwd, TempDir}]),
+    {ok, [Name]} = zip:extract(Archive, [verbose]),
 
-    %% Verify that absolute file was not created
-    {error, enoent} = file:read_file(Name),
-
-    %% Verify that relative contents of extracted file.
-    {ok, Bin} = file:read_file(TempRelName),
+    %% Verify contents of extracted file.
+    {ok, Bin} = file:read_file(Name),
     true = match_byte_list(X0, binary_to_list(Bin)),
 
+
     %% Verify that Unix zip can read it. (if we have a unix zip that is!)
-    zipinfo_match(Archive, RelName),
+    zipinfo_match(Archive, Name),
 
     ok.
 
@@ -1060,21 +1056,10 @@ run_command(Command, Args) ->
              end
      end)().
     
-sanitize_filenames(Config) ->
-    RootDir = proplists:get_value(priv_dir, Config),
-    TempDir = filename:join(RootDir, "borderline"),
-    ok = file:make_dir(TempDir),
-
-    %% Create a zip archive /tmp/absolute in it
-    %%   This file was created using the command below on Erlang/OTP 28.0
-    %%   1> rr(file), {ok, {_, Bin}} = zip:zip("absolute.zip", [{"/tmp/absolute",<<>>,#file_info{ type=regular, mtime={{1970,1,1},{0,0,0}}, size=0 }}], [memory]), rp(base64:encode(Bin)).
-    AbsZip = base64:decode(<<"UEsDBBQAAAAAAAAAIewAAAAAAAAAAAAAAAANAAAAL3RtcC9hYnNvbHV0ZVBLAQIUAxQAAAAAAAAAIewAAAAAAAAAAAAAAAANAAAAAAAAAAAAAACkAQAAAAAvdG1wL2Fic29sdXRlUEsFBgAAAAABAAEAOwAAACsAAAAAAA==">>),
-    Archive = filename:join(TempDir, "absolute.zip"),
-    ok = file:write_file(Archive, AbsZip),
-
-    TmpAbs = filename:join([TempDir, "tmp", "absolute"]),
-    {ok, [TmpAbs]} = zip:unzip(Archive, [verbose, {cwd, TempDir}]),
-    {error, enoent} = file:read_file("/tmp/absolute"),
-    {ok, <<>>} = file:read_file(TmpAbs),
-
-    ok.
\ No newline at end of file
+explicit_file_info(_Config) ->
+    Epoch = {{1980,1,1},{0,0,0}},
+    FileInfo = #file_info{type=regular, size=0, mtime=Epoch},
+    Files = [{"datetime", <<>>, FileInfo},
+             {"seconds", <<>>, FileInfo#file_info{mtime=315532800}}],
+    {ok, _} = zip:zip("", Files, [memory]),
+    ok.
