@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2001-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2001-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -33,7 +33,7 @@ all() ->
                    analyse_no_beam, line_0, compile_beam_no_file,
                    compile_beam_missing_backend,
                    otp_13277, otp_13289, guard_in_lc, gh_4796,
-                   eep49],
+                   eep49, gh_8159],
     StartStop = [start, compile, analyse, misc, stop,
                  distribution, reconnect, die_and_reconnect,
                  dont_reconnect_after_stop, stop_node_after_disconnect,
@@ -864,7 +864,6 @@ export_import(Config) when is_list(Config) ->
     %% warning is written when data is deleted for imported module.
     test_server:capture_start(),
     {ok,f} = cover:compile(f),
-    timer:sleep(10), % capture needs some time
     [Text3] = test_server:capture_get(),
     "WARNING: Deleting data for module f imported from" ++ _ = lists:flatten(Text3),
     test_server:capture_stop(),
@@ -1587,9 +1586,7 @@ otp_14817(Config) when is_list(Config) ->
     ok = otp_14817:b(),
     ok = otp_14817:c(),
     ok = otp_14817:d(),
-    {ok,[{{otp_14817,3},1},
-         {{otp_14817,3},1},
-         {{otp_14817,3},1},
+    {ok,[{{otp_14817,3},3},
          {{otp_14817,4},1}]} =
         cover:analyse(otp_14817, calls, line),
     {ok, CovOut} = cover:analyse_to_file(otp_14817),
@@ -1900,9 +1897,9 @@ eep49(Config) ->
 
     File = "t.erl",
     Test = <<"-module(t).
-              -feature(maybe_expr,enable).
               -export([t/0]).
 
+
               t() ->
                   t1(),                         %6
 
@@ -1950,6 +1947,50 @@ eep49(Config) ->
     ok = file:delete(File),
     ok.
 
+gh_8159(Config) ->
+    ok = file:set_cwd(proplists:get_value(priv_dir, Config)),
+
+    M = same_line,
+    File = atom_to_list(M) ++ ".erl",
+    Test = ~"""
+            -module(same_line).
+            -export([aaa/0, bbb/0, ccc/0]).
+            bbb() -> ok. aaa() -> not_ok. ccc() -> cool.
+            """,
+    ok = file:write_file(File, Test),
+    {ok, M} = cover:compile(File),
+    {ok,[{{M,aaa,0},0}, {{M,bbb,0},0}, {{M,ccc,0},0}]} = cover:analyse(M, calls, function),
+    {ok,[{{M,3},0}]} = cover:analyse(M, calls, line),
+    {ok,[{{M,3},{0,1}}]} = cover:analyse(M, coverage, line),
+
+    cool = M:ccc(),
+    {ok,[{{M,aaa,0},0}, {{M,bbb,0},0}, {{M,ccc,0},1}]} = cover:analyse(M, calls, function),
+    {ok,[{{M,3},1}]} = cover:analyse(M, calls, line),
+    {ok,[{{M,3},{1,0}}]} = cover:analyse(M, coverage, line),
+
+    not_ok = M:aaa(),
+    {ok,[{{M,aaa,0},1}, {{M,bbb,0},0}, {{M,ccc,0},1}]} = cover:analyse(M, calls, function),
+    {ok,[{{M,3},2}]} = cover:analyse(M, calls, line),
+    {ok,[{{M,3},{1,0}}]} = cover:analyse(M, coverage, line),
+
+    ok = M:bbb(),
+    {ok,[{{M,aaa,0},1}, {{M,bbb,0},1}, {{M,ccc,0},1}]} = cover:analyse(M, calls, function),
+    {ok,[{{M,3},3}]} = cover:analyse(M, calls, line),
+    {ok,[{{M,3},{1,0}}]} = cover:analyse(M, coverage, line),
+
+    not_ok = M:aaa(),
+    {ok,[{{M,aaa,0},2}, {{M,bbb,0},1}, {{M,ccc,0},1}]} = cover:analyse(M, calls, function),
+    {ok,[{{M,3},{1,0}}]} = cover:analyse(M, coverage, line),
+
+    cover:reset(),
+
+    not_ok = M:aaa(),
+    {ok,[{{M,3},1}]} = cover:analyse(M, calls, line),
+
+    ok = file:delete(File),
+
+    ok.
+
 
 %%--Auxiliary------------------------------------------------------------
 
